package com.wanji.indicators.cache;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.io.ParseException;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;

import java.io.*;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author fengyi
 * @date 2023/10/31
 * @description
 */
@Slf4j
public class PedestrianDataCache {
    //安全岛 key:crossId_key value：围栏坐标
    private static Map<String, String> safetyIslandMap = new ConcurrentHashMap<>();
    private static Map<String, String> crossingWalkMap = new ConcurrentHashMap<>();

    private static PedestrianDataCache instance;

    private PedestrianDataCache() {
    }

    public static synchronized PedestrianDataCache getInstance() {
        if (instance == null) {
            instance = new PedestrianDataCache();
            String f1 = "/data/gis-aqd.geojson";
            String f2 = "/data/gis-rxhd.geojson";
            instance.loadData(f1, 1);
            instance.loadData(f2, 2);
        }
        return instance;
    }

    private Map<String, String> loadData(String file, int type) {
        Map<String, String> retMap = new HashMap<>();

        String filePath = PedestrianDataCache.class.getResource(file).getPath();

        try (FileReader fr = new FileReader(filePath);
             BufferedReader br = new BufferedReader(fr);
        ) {
            StringBuilder jsonBuilder = new StringBuilder();
            String str = null;
            // 按行读取字符串
            while ((str = br.readLine()) != null) {
                jsonBuilder.append(str);
            }
            if (jsonBuilder.length() > 0) {
                JSONObject jsonObject = JSON.parseObject(jsonBuilder.toString());
                JSONArray jsonArray = jsonObject.getJSONArray("features");
                int size = jsonArray.size();
                for (int i = 0; i < size; i++) {
                    JSONObject jsonObject1 = jsonArray.getJSONObject(i);
                    JSONObject propJsonObj = jsonObject1.getJSONObject("properties");
                    String crossId = propJsonObj.getString("lkid");
                    String rid = propJsonObj.getString("ldid");

                    JSONObject geometryJsonObj = jsonObject1.getJSONObject("geometry");
                    JSONArray coordinatesArray = geometryJsonObj.getJSONArray("coordinates");

                    StringBuilder stringBuilder = new StringBuilder();
                    for (int j = 0; j < coordinatesArray.size(); j++) {
                        JSONArray pointArr = coordinatesArray.getJSONArray(j);
                        for (int k=0; k<pointArr.size();k++) {
                            double lng = pointArr.getJSONArray(k).getDouble(0);
                            double lat = pointArr.getJSONArray(k).getDouble(1);
                            stringBuilder.append(lng + "," + lat);
                            if (k != pointArr.size() - 1) {
                                stringBuilder.append(";");
                            }
                        }
                    }
                    //Geometry geometry = GeomsConvertUtil.genGeometry(stringBuilder.toString());
                    String key = rid;
                    if (type == 1) {

                        GeomsConvertUtil.genGeometry(stringBuilder.toString());

                        instance.safetyIslandMap.put(key, stringBuilder.toString());
                    } else if (type == 2) {
                        instance.crossingWalkMap.put(key, stringBuilder.toString());
                    }
                }
            }
        } catch (Exception e) {
            log.error("", e);
        }
        return retMap;
    }

    public Map<String, String> getSafetyIslandMap() {
        return safetyIslandMap;
    }

    public Map<String, String> getCrossingWalkMap() {
        return crossingWalkMap;
    }

    /**
     * 通过区域相交获取所属rid
     *
     * @param lng
     * @param lat
     * @return
     */
    public String getRidByBmxGeomIntersect(double lng, double lat) {
        for (Map.Entry<String, String> entry : instance.crossingWalkMap.entrySet()) {
            String rid = entry.getKey();
            String areaPoints = entry.getValue();
            boolean isIn = GeomsConvertUtil.withinGeo(lng + "," + lat, areaPoints);
            if (isIn) {
                return rid;
            }
        }
        return null;
    }

    /**
     * 通过区域相交获取所属rid
     *
     * @param lng
     * @param lat
     * @return
     */
    public String getRidByAqdGeomIntersect(double lng, double lat) {
        for (Map.Entry<String, String> entry : instance.safetyIslandMap.entrySet()) {
            String rid = entry.getKey();
            String areaPoints = entry.getValue();
            boolean isIn = GeomsConvertUtil.withinGeo(lng + "," + lat, areaPoints);
            if (isIn) {
                return rid;
            }
        }
        return null;
    }
}
