package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.EventAbnormalEnum;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

public class AccidentCheckTimeFlatMap implements FlatMapFunction<AccidentFrameModel, AccidentFrameModel> {

    private static final Logger log = LoggerFactory.getLogger(AccidentCheckTimeFlatMap.class);

    @Override
    public void flatMap(AccidentFrameModel accidentFrameModel, Collector<AccidentFrameModel> collector) throws Exception {
        if (accidentFrameModel.getConfigModel().getAccidentIsOpen() == Constant.OPEN) {
            List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();
            List<AccidentFrameModel.CarTrack> listAccident = new ArrayList<>();
            List<AccidentFrameModel.CarTrack> listStop = new ArrayList<>();
            for (AccidentFrameModel.CarTrack carTrack : trackList) {
//                if (carTrack.getGlobalTimeStamp()>=1697265909000L && "鲁AD66123".equals(carTrack.getPicLicense())){
//                    System.out.println();
//                }
                if (carTrack.isStop() && carTrack.isHasSameTurnCar()) {
                    //计算停车时间阈值
                    long checkTime = getCheckTime(carTrack);
                    carTrack.setCheckTime(checkTime);
                    //判断停车时间是否超过阈值 超过阈值则加入疑似事故车辆集合
                    if (carTrack.getStopDuration() > accidentFrameModel.getConfigModel().getAbnormalParkStopBaseDuration()) {
                        //加入疑似事故集合
                        listAccident.add(carTrack);
                    }
                    //加入停驶车辆集合
                    listStop.add(carTrack);
                }
            }
            //路口内停驶车辆组合事故
            List<AccidentFrameModel.CarTrack> carTracks = checkDis(listAccident, listStop, accidentFrameModel.getConfigModel());
            int size = carTracks.size();
            if (size > 0) {
                accidentFrameModel.setTrackList(carTracks);
                if (size == 1) {
                    //单车事故
                    accidentFrameModel.setAccidentType(EventAbnormalEnum.ACCIDENT_ONE_YS.getType());
                } else if (size == 2) {
                    //双车事故
                    accidentFrameModel.setAccidentType(EventAbnormalEnum.ACCIDENT_TWO_YS.getType());
                } else {
                    //多车事故
                    accidentFrameModel.setAccidentType(EventAbnormalEnum.ACCIDENT_MORE_YS.getType());
                }
                collector.collect(accidentFrameModel);
            } else {
                //单车事故
                accidentFrameModel.setTrackList(listAccident);
                accidentFrameModel.setAccidentType(EventAbnormalEnum.ACCIDENT_ONE_YS.getType());
                collector.collect(accidentFrameModel);
            }
        }
    }


    /**
     * 计算车辆之间的距离是否达到要求
     *
     * @param listAccident
     * @param listStop
     * @return
     */
    private List<AccidentFrameModel.CarTrack> checkDis(List<AccidentFrameModel.CarTrack> listAccident, List<AccidentFrameModel.CarTrack> listStop, ConfigModel configModel) {
        List<AccidentFrameModel.CarTrack> list = new ArrayList<>();
        Set<String> set = new HashSet<>();
        for (AccidentFrameModel.CarTrack accident : listAccident) {
            int trackIdA = accident.getId();
            long stopTimeA = accident.getStopTime();

            double distanceMin = Double.MAX_VALUE;
            for (AccidentFrameModel.CarTrack stop : listStop) {
                int trackIdS = stop.getId();
                long stopTimeS = stop.getStopTime();
                //排除当前车辆
                if (trackIdA != trackIdS) {
                    double distance = PtInPolyUtil.getDistance(accident.getLongitude(), accident.getLatitude(), stop.getLongitude(), stop.getLatitude());
                    if (distance < 10.0) {
                        //小于10米
                        long stopDu = stop.getStopDuration();
                        //疑似事故车辆与停车车辆之间的停车时间差
                        long tx = Math.abs(stopTimeA - stopTimeS);
                        boolean isOK = false;
                        if (distance < 10.0 && tx < 15000L && stopDu > 7500L) {
                            //时间差小于15秒 且 匹配车辆停车时间大于7.5秒的则同意匹配
                            log.info("LOG_EVENT_ACCIDENT_MERGE >>> 10 > 车辆合并 停车k:" + stop.getPicLicense() + "-" + stop.getId() +
                                    " 跟踪k:" + accident.getPicLicense() + "-" + accident.getId() +
                                    " 停车时间差:" + tx + " 距离:" + distance + "  停车时间:" + stopDu);
                            isOK = true;
                        } else if (distance < 9.0 && tx < 30000L && stopDu > 20000L) {
                            //时间差小于30秒 且 匹配车辆停车时间大于20秒的则同意匹配
                            log.info("LOG_EVENT_ACCIDENT_MERGE >>> 9 > 车辆合并 停车k:" + stop.getPicLicense() + "-" + stop.getId() +
                                    " 跟踪k:" + accident.getPicLicense() + "-" + accident.getId() +
                                    " 停车时间差:" + tx + " 距离:" + distance + "  停车时间:" + stopDu);
                            isOK = true;
                        } else if (distance < 8.0 && tx < 180000L && stopDu > 30000L && stopDu < 190000L) {
                            log.info("LOG_EVENT_ACCIDENT_MERGE >>> 8 > 车辆合并 停车k:" + stop.getPicLicense() + "-" + stop.getId() +
                                    " 跟踪k:" + accident.getPicLicense() + "-" + accident.getId() +
                                    " 停车时间差:" + tx + " 距离:" + distance + "  停车时间:" + stopDu);
                            isOK = true;
                        }
                        if (stopDu >= configModel.getAbnormalParkStopBaseDuration()) {
                            if (set.add(trackIdA + "-" + trackIdS) && set.add(trackIdS + "-" + trackIdA)) {
                                //如果距离小于上一车辆
                                if (distanceMin > distance) {
                                    distanceMin = distance;
                                    accident.setAccidentTrackId(stop.getId());
                                    accident.setAccidentDis(distance);
                                    stop.setAccidentTrackId(accident.getId());
                                    stop.setAccidentDis(distance);
                                    list.add(accident);
                                    list.add(stop);
//                                    System.err.println(trackIdA + "-"+accident.getPicLicense() + " >> " + trackIdS +"-"+ stop.getPicLicense() +
//                                            "  dis:" + distance + "  stA: " + stopTimeA + "  stS: " + stop.getStopTime() +
//                                            "  tx:" + tx + "  accidentDu:" + accident.getStopDuration() +
//                                            "  stopDu:" + stop.getStopDuration());
                                }
                            }
                        }
                    }
                }
            }
        }
        if (!list.isEmpty()) {
            list = sort(list);
        }
        return list;
    }

    /**
     * 计算停车时间阈值
     *
     * @param carTrack
     * @return
     */
    private long getCheckTime(AccidentFrameModel.CarTrack carTrack) {
        String laneFunc = TurnEnum.getLaneFunction(carTrack.getRoadnet().getTurn());
        // 停车时间阈值 = 最大停车时间阈值 - TTC减免时间 - 制动减免时间 - 绕行减免时间 - 停车区域减免时间 + 左转待转区
        long base_time = 10000;
        long base_min_time = 60000;
        long base_max_time = 210000;
        long ttc_time = 0;
        long at_time = 0;
        long speed_time = 0;
        //初始化时间阈值 取最大验证时间
        long checkTime = base_max_time;
        //获取绕行减免时间
        long detour_time = carTrack.getDetourDuration() + (carTrack.getDetourNum() * 1000);
        if (detour_time > 60000) {
            detour_time = 60000;
        }
        //根据加速度计算加速度减免时间
        if (carTrack.isEmergencyBraking()) {
            at_time = new Double(base_time * Math.abs(carTrack.getEmergencyBrakingAt())).longValue();
        }
        //根据刹车前速度计算减免时间
        if (carTrack.isEmergencyBraking()) {
            speed_time = new Double(1000 * carTrack.getEmergencyBrakingSpeed()).longValue();
        }
        //根据TTC计算TTC减免时间
        if (carTrack.isTTC() && carTrack.getAreaDist() > 5.0) {
            ttc_time = base_time + new Double(1000 * carTrack.getTtcSpeed()).longValue();
        }
        //车辆在绿灯情况下还没开走
        long greenStopDuration = carTrack.getGreenStopDuration();
        //根据停车区域获取停车减免时间
        long area_time = getStopCheckTimeByAreaDis(carTrack);
        //去除减免时间计算最终阈值   停车区域减免    绕行减免      制动减免    ttc减免    刹车前车速减免   绿灯停车持续时间减免
        checkTime = checkTime - area_time - detour_time - at_time - ttc_time - speed_time - greenStopDuration;
        if (checkTime < base_min_time) {
            checkTime = base_min_time;
        }
        //车辆在待转区内 则验证时间增加
        if (carTrack.getWaitLaneDistance() >= carTrack.getAreaDist()) {
            checkTime = checkTime + detour_time + base_time;
        } else if (StringUtils.isNotBlank(laneFunc) && laneFunc.contains("1")) {
            checkTime = checkTime + Math.abs(area_time);
        }
        //判断是否有灯态  有则使用灯态  无责直接使用无灯态的逻辑
        if (false/*StringUtils.isNotBlank(carTrack.getLampState()) && !"null".equals(carTrack.getLampState())*/) {
            log.info("LOG_EVENT_ACCIDENT_CHECK_TIME >>> HAVE LAMP STATE " + carTrack.getRoadnet().getCrossId() +
                    "-" + carTrack.getId() + "-" + carTrack.getPicLicense() +
                    " stopDu: " + carTrack.getStopDuration() + "  lampState: "/* + carTrack.getLampState()*/ +
                    " GreenStopDuration: " + carTrack.getGreenStopDuration());
            //车辆在绿灯情况下还没开走  持续8秒后直接输出
            if (carTrack.getGreenStopDuration() > 30000L) {
                return checkTime;
            } else {
                return base_max_time + base_min_time + base_time;
            }
        } else {
            if (checkTime > base_min_time) {
                log.info("LOG_EVENT_ACCIDENT_CHECK_TIME >>> NO HAVE LAMP STATE " + carTrack.getRoadnet().getCrossId() +
                        "-" + carTrack.getId() + "-" + carTrack.getPicLicense() + " checkTime: " + checkTime +
                        " stopDu: " + carTrack.getStopDuration() + " 区: " + area_time +
                        " 绕:" + detour_time + " detDu: " + carTrack.getDetourDuration() +
                        " 减:" + at_time + " TTC: " + ttc_time + " areaDis: " + carTrack.getAreaDist() +
                        " at:" + carTrack.getEmergencyBrakingAt() + " speed_time: " + speed_time +
                        " speed: " + carTrack.getEmergencyBrakingSpeed() + " 稳:" + carTrack.getStopStableDuration() +
                        " green_stop: " + greenStopDuration +
                        " 时间: " + DateFormatUtils.format(carTrack.getStopTime(), DateUtil.YYYY_MM_DD_HH_MM_SS) +
                        " laneFunc: " + laneFunc +
                        " hashLampState: " /*+ carTrack.getLampState()*/);
            }
            return checkTime;
        }
    }

    /**
     * 根据停车区域计算停车减免时间
     *
     * @param carTrack
     * @return
     */
    private long getStopCheckTimeByAreaDis(AccidentFrameModel.CarTrack carTrack) {
        long ratio = 2000;
        double areaDist = carTrack.getAreaDist();
        if (carTrack.getRoadnet().getInCrossFlag() == Constant.CROSSING) {
            //距离接近停车线的则减小倍率
            if (areaDist < 18.0 && areaDist >= 15.0) {
                ratio = 1800;
            } else if (areaDist < 15.0 && areaDist >= 13.0) {
                ratio = 1600;
            } else if (areaDist < 13.0 && areaDist >= 11.0) {
                ratio = 1200;
            } else if (areaDist < 11.0 && areaDist >= 9.0) {
                ratio = 600;
            } else if (areaDist < 9.0 && areaDist >= 6.0) {
                ratio = 0;
            } else if (areaDist < 6.0 && areaDist >= 3.0) {
                ratio = -5000;
                areaDist = 6.0 - areaDist;
            } else if (areaDist < 3.0 && areaDist >= 1.0) {
                ratio = -10000;
                areaDist = 3.0 - areaDist;
            } else if (areaDist < 1.0) {
                ratio = -100000;
                areaDist = 1.0 - areaDist;
                if (areaDist < 0.2) {
                    areaDist = 0.2;
                }
            }
            long l = new Double(areaDist * ratio).longValue();
            if (l > 90000) {
                l = 90000;
            }
            return l;
        } else {
            return 60000;
        }
    }


    /**
     * 排序 -- 时间戳 -- 升序
     *
     * @param list
     * @return
     */
    public List<AccidentFrameModel.CarTrack> sort(List<AccidentFrameModel.CarTrack> list) {
        try {
            Collections.sort(list, new Comparator<AccidentFrameModel.CarTrack>() {
                @Override
                public int compare(AccidentFrameModel.CarTrack o1, AccidentFrameModel.CarTrack o2) {
                    long o1x = o1.getId();
                    long o2x = o2.getId();
                    if (o1x > o2x) {
                        return 1;
                    }
                    if (o1x == o2x) {
                        return 1;
                    }
                    return -1;
                }
            });
            return list;
        } catch (Exception e) {
            return list;
        }
    }

    /**
     * 计算行车角度是否同向
     *
     * @param angleA
     * @param angleB
     * @return
     */
    private static boolean isSyntropy(double angleA, double angleB) {
        double angleX = Math.abs(angleA - angleB);
        if (angleX > 180.0) {
            angleX = 360.0 - angleX;
        }
        if (angleX <= 60.0) {
            return true;
        } else {
            return false;
        }
    }


}
