package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.EventAbnormalEnum;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.util.DateUtil;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 *
 */
public class AccidentDuplicateRemoveKeyedProcess extends KeyedProcessFunction<String, AccidentFrameModel, AccidentFrameModel> {

    private static final Logger log = LoggerFactory.getLogger(AccidentDuplicateRemoveKeyedProcess.class);

    private MapState<String, AccidentFrameModel.CarTrack> mapState;

    @Override
    public void open(Configuration parameters) {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("accidentQCMapState_01011101", String.class, AccidentFrameModel.CarTrack.class));
    }

    @Override
    public void processElement(AccidentFrameModel accidentFrameModel, Context context, Collector<AccidentFrameModel> collector) throws Exception {
        String accidentType = accidentFrameModel.getAccidentType();
        List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();
        boolean isOk = true;
//        AccidentFrameModel distinctModel = new AccidentFrameModel();
//        BeanUtils.copyProperties(accidentFrameModel,distinctModel);

        List<AccidentFrameModel.CarTrack> distinctCarList = new ArrayList<>();

        for (AccidentFrameModel.CarTrack carTrack : trackList) {
            int trackId = carTrack.getId();
            String crossId = carTrack.getRoadnet().getCrossId();
            String key = crossId + Constant.MARK + trackId + Constant.MARK + accidentType;
            //AccidentFrameModel.CarTrack value = mapState.get(key);
            if (isOk) {
                AccidentFrameModel.CarTrack value = mapState.get(key);
                if (value == null) {
                    mapState.put(key, carTrack);
                } else {
                    isOk = false;
                }
            }
            if (!EventAbnormalEnum.ACCIDENT_ONE_YS.getType().equals(accidentType)) {
                key = crossId + Constant.MARK + trackId + Constant.MARK + EventAbnormalEnum.ACCIDENT_ONE_YS.getType();
                AccidentFrameModel.CarTrack valueCheck = mapState.get(key);
                if (valueCheck != null) {
                    mapState.remove(key);
                    isOk = true;
                }
            }
        }
        if (isOk) {
//            Iterator<Map.Entry<String, AccidentFrameModel.CarTrack>> iterator = mapState.iterator();
//            while (iterator.hasNext()) {
//                Map.Entry<String, AccidentFrameModel.CarTrack> entry = iterator.next();
//                String akey = entry.getKey();
//                AccidentFrameModel.CarTrack carTrack1 = entry.getValue();
//                System.out.println();
//            }
//            distinctModel.setTrackList(distinctCarList);
            //输出结果数据
            collector.collect(accidentFrameModel);
            //日志记录
            logInfo(accidentFrameModel);
            //清除状态缓存
            clearStateCache(accidentFrameModel);
        }
    }

    /**
     * 定期清除状态中的数据
     *
     * @param accidentFrameModel
     * @throws Exception
     */
    private void clearStateCache(AccidentFrameModel accidentFrameModel) throws Exception {
        Iterator<AccidentFrameModel.CarTrack> iterator = mapState.values().iterator();
        List<AccidentFrameModel.CarTrack> clear = new ArrayList<>();
        while (iterator.hasNext()) {
            AccidentFrameModel.CarTrack next = iterator.next();
            clear.add(next);
        }
        //遍历并清除事故
        for (AccidentFrameModel.CarTrack carTrack1 : clear) {
            long x = Math.abs(carTrack1.getGlobalTimeStamp() - accidentFrameModel.getTime());
            if (x > 3600000) {
                mapState.remove(carTrack1.getRoadnet().getCrossId() + Constant.MARK + carTrack1.getId() + Constant.MARK + EventAbnormalEnum.ACCIDENT_ONE_YS.getType());
                mapState.remove(carTrack1.getRoadnet().getCrossId() + Constant.MARK + carTrack1.getId() + Constant.MARK + EventAbnormalEnum.ACCIDENT_MORE_YS.getType());
                mapState.remove(carTrack1.getRoadnet().getCrossId() + Constant.MARK + carTrack1.getId() + Constant.MARK + EventAbnormalEnum.ACCIDENT_TWO_YS.getType());
            }
        }
    }

    /**
     * 记录日志
     *
     * @param accidentFrameModel
     */
    private void logInfo(AccidentFrameModel accidentFrameModel) {
        List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();
        boolean isOk = false;
        String log1 = "";
        String log2 = "";
        for (AccidentFrameModel.CarTrack carTrack : trackList) {
            //绕行时间
            long detourDuration = carTrack.getDetourDuration();
            //停车时间
            long stopDuration = carTrack.getStopDuration();
            boolean stop = carTrack.isStop();
            if (stop) {
                log1 += "==> " + carTrack.getId() + "-" + carTrack.getPicLicense() + " " + DateFormatUtils.format(carTrack.getStopTime(), DateUtil.YYYY_MM_DD_HH_MM_SS) + " ";
                log2 += " | " + carTrack.getPicLicense() + "  du:" + stopDuration +
                        " rao:" + detourDuration + " dis:" + carTrack.getAreaDist() + " num:" + carTrack.getDetourNum() +
                        " ddu:" + carTrack.getDetourDuration() + " " +
                        DateFormatUtils.format(carTrack.getStopTime(), DateUtil.YYYY_MM_DD_HH_MM_SS) +
                        " braking: " + carTrack.isEmergencyBraking();
                isOk = true;
            }
        }
        if (isOk) {
            log.info("LOG_EVENT_ACCIDENT_DUPLICATE >>> " + accidentFrameModel.getAccidentType() + " ==>> " + log1 + " ==>> " + log2);
        }
    }

}
