package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.util.Collector;
import org.springframework.beans.BeanUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 *
 * @auther fengyi
 */
public class AccidentFlatMap implements FlatMapFunction<CrossFrameModel, AccidentFrameModel> {
    @Override
    public void flatMap(CrossFrameModel frameModel, Collector<AccidentFrameModel> collector) throws Exception {
        List<CarTrackModel> trackList = frameModel.getTrackList();
        List<AccidentFrameModel.CarTrack> list = new ArrayList<>();

        ConfigModel configModel = frameModel.getConfigModel();
        if (configModel.getAbnormalParkIsOpen() == Constant.OPEN) {
            for (CarTrackModel carTrack : trackList) {
                String crossId = carTrack.getRoadnet().getCrossId();
                if (!"13NI00B5RM0".equals(crossId)) {
                    continue;
                }

                boolean isMotor = VehicleTypeEnum.isMotorVehicles(carTrack.getOriginalType().shortValue());
                int inCrossingFlag = carTrack.getRoadnet().getInCrossFlag();
                //路口内机动车
                if (isMotor && inCrossingFlag==Constant.CROSSING) {
                    AccidentFrameModel.CarTrack car = new AccidentFrameModel.CarTrack();
                    BeanUtils.copyProperties(carTrack, car);
                    String laneEndPoint = car.getRoadnet().getLaneEndPoint();
                    if (Objects.isNull(laneEndPoint)){
                        continue;
                    }
//                    if (carTrack.getGlobalTimeStamp()>1697265909793L && "鲁AD66123".equals(carTrack.getPicLicense())){
//                        System.out.println();
//                    }
                    String[] sps = laneEndPoint.split(",");
                    Double angeleToLane = PtInPolyUtil.getAngle(carTrack.getLongitude(),carTrack.getLatitude(),Double.parseDouble(sps[0]),Double.parseDouble(sps[1]));
                    //通过车辆与车道终点坐标得出的角度与车道方位角度判断是否驶出停止线
                    double angleX = Math.abs(angeleToLane - carTrack.getRoadnet().getLaneAngle());
                    boolean isReverseDir = false;
                    if (angleX > 160.0 && angleX < 200.0) {
                        isReverseDir = true;
                    }
                    if (isReverseDir) {
                        //距离停止线距离
                        double stopLineDist = GeomsConvertUtil.getDistance(car.getLongitude(), car.getLatitude(), Double.parseDouble(sps[0]), Double.parseDouble(sps[1]));
                        car.setStopLineDist(stopLineDist);
                    }
                    list.add(car);
                    //筛选行驶越过斑马线
//                    if (car.getStopLineDist()> configModel.getAbnormalParkZebraCrossLength()) {
//                        String  dt = DateUtil.toDateTime(car.getGlobalTimeStamp(),DateUtil.YYYY_MM_DD_HH_MM_SS);
////                        if (carTrack.getGlobalTimeStamp()>1697265909793L && "鲁AD66123".equals(carTrack.getPicLicense())){
////                            System.out.println();
////                        }
//                        list.add(car);
//                    }
                }
            }
            AccidentFrameModel model = new AccidentFrameModel();
            model.setCrossId(frameModel.getCrossId());
            model.setTime(frameModel.getGlobalTimeStamp());
            model.setTrackList(list);
//            model.setRidTurnLampList(frameModel.getRidTurnLampList());
//            model.setConfigModel(frameModel.getConfigModel());
            collector.collect(model);
        }

    }
}
