package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.accident.AccidentDetourModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @auther fengyi
 */
public class AccidentSameDriDriveKeyedProcess extends KeyedProcessFunction<String, AccidentFrameModel, AccidentFrameModel> {

    private static final Logger log = LoggerFactory.getLogger(AccidentSameDriDriveKeyedProcess.class);

    private MapState<String, AccidentDetourModel> mapState;
    private MapState<String, AccidentDetourModel> mapStateLike;
    private MapState<String, AccidentFrameModel.CarTrack> mapStateDuration;

    @Override
    public void open(Configuration parameters) throws Exception {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateCheckSameDriDrive_01x010X", String.class, AccidentDetourModel.class));
        mapStateLike = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateCheckSameDriDrive_01x011X", String.class, AccidentDetourModel.class));
        mapStateDuration = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateCheckSameDriDriveDuration_01x012X", String.class, AccidentFrameModel.CarTrack.class));
    }

    @Override
    public void processElement(AccidentFrameModel accidentFrameModel, Context context, Collector<AccidentFrameModel> collector) throws Exception {
        ConfigModel configModel = accidentFrameModel.getConfigModel();

        List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();

        List<AccidentFrameModel.CarTrack> trackRun = trackList.stream().filter(o->!o.isStop()).collect(Collectors.toList());
        Map<String,List<AccidentFrameModel.CarTrack>> runGroup = trackRun.stream().collect(Collectors.groupingBy(o->o.getRoadnet().getRid()));

        List<AccidentFrameModel.CarTrack> trackStop = trackList.stream().filter(o->o.isStop()).collect(Collectors.toList());
        Map<String,List<AccidentFrameModel.CarTrack>> stopGroup = trackStop.stream().collect(Collectors.groupingBy(o->o.getRoadnet().getRid()));


        List<AccidentFrameModel.CarTrack> list = new ArrayList<>();

        for (Map.Entry<String,List<AccidentFrameModel.CarTrack>> entry : stopGroup.entrySet()){
            String rid = entry.getKey();
            //同方向上运行的车辆
            List<AccidentFrameModel.CarTrack> runCarList = runGroup.get(rid);
            //同方向上停车车辆
            List<AccidentFrameModel.CarTrack> stopCarList = entry.getValue();
            for (AccidentFrameModel.CarTrack carTrack : stopCarList) {
                String laneFunc = TurnEnum.getLaneFunction(carTrack.getRoadnet().getTurn());
                String[] sps = laneFunc.split(Constant.COMMA);
                boolean isHasSameTurnCar = false;
                //遍历车道功能【混合车道有多个转向】
                for (String func : sps) {
                    try {
                        if (Objects.nonNull(runCarList)) {
                            List<AccidentFrameModel.CarTrack> sameTurnCarList = runCarList.stream().filter(o -> TurnEnum.getLaneFunction(o.getRoadnet().getTurn()).contains(func)).collect(Collectors.toList());
                            //判断停车车辆是否有从相同转向运行的车辆，有同向行驶车辆加入到列表传递给下一个算子
                            if (!sameTurnCarList.isEmpty()) {
                                isHasSameTurnCar = true;
                                break;
                            }
                        }
                    }catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                carTrack.setHasSameTurnCar(isHasSameTurnCar);
                list.add(carTrack);

            }
        }

        //发送数据
        if (!list.isEmpty()) {
            accidentFrameModel.setTrackList(list);
            collector.collect(accidentFrameModel);
        }

    }


    /**
     * 验证绕行
     *
     * @param accidentFrameModel
     * @param trackStop
     * @param trackRun
     * @throws Exception
     */
    private void checkDetourFlow(ConfigModel configModel, AccidentFrameModel accidentFrameModel, List<AccidentFrameModel.CarTrack> trackStop, List<AccidentFrameModel.CarTrack> trackRun) throws Exception {
        long timeNow = accidentFrameModel.getTime();
        for (AccidentFrameModel.CarTrack carStop : trackStop) {
            double lngS = carStop.getLongitude();
            double latS = carStop.getLatitude();
            String ridS = carStop.getRoadnet().getRid();
            Integer laneTurnFunc = carStop.getRoadnet().getTurn();
            //车道功能组装
            String laneFuncS = TurnEnum.getLaneFunction(laneTurnFunc);

            for (AccidentFrameModel.CarTrack carRun : trackRun) {
                double lngR = carRun.getLongitude();
                double latR = carRun.getLatitude();
                String ridR = carRun.getRoadnet().getRid();
                String laneFuncR = TurnEnum.getLaneFunction(carRun.getRoadnet().getTurn());

                if (StringUtils.isNotBlank(ridS) && ridS.equals(ridR) &&
                        StringUtils.isNotBlank(laneFuncS) && StringUtils.isNotBlank(laneFuncR) &&
                        (laneFuncS.contains(laneFuncR) || laneFuncR.contains(laneFuncS))) {
                    //计算当前帧停驶车辆与行驶车辆之间的距离
                    double distance = PtInPolyUtil.getDistance(lngS, latS, lngR, latR);
                    double driveAngleS = carStop.getCourseAngle();
                    double driveAngleR = carRun.getCourseAngle();
                    String key = getKey(carStop, carRun);
                    String keyLike = getKeyLike(carStop);

                    AccidentDetourModel detourNow = new AccidentDetourModel();
                    detourNow.setKey(key);
                    detourNow.setTime(timeNow);
                    detourNow.setDistance(distance);
                    detourNow.setDuration(0L);

                    AccidentDetourModel detourLikeNow = new AccidentDetourModel();
                    detourLikeNow.setKey(keyLike);
                    detourLikeNow.setTime(timeNow);
                    detourLikeNow.setDistance(distance);
                    detourLikeNow.setDuration(0L);

                    //查询上一帧两车的距离
                    AccidentDetourModel detourAgo = mapState.get(key);
                    AccidentDetourModel detourDurationAgo = mapStateLike.get(keyLike);
                    if (detourDurationAgo != null) {
                        //计算两车行驶方向是否相同
                        boolean syntropy = isSyntropy(driveAngleS, driveAngleR, 45.0);
                        if (distance < configModel.getAbnormalParkDetourDistanceAfterStopCar() && syntropy) {
                            long durationReal = detourDurationAgo.getDuration();
                            long timeX = Math.abs(timeNow - detourDurationAgo.getTime());
                            //距离大于0 且 当前距离要比上一帧距离大 (表示绕行车辆正在远离目标)
                            if (detourAgo != null && distance > 0.0 && distance > detourAgo.getDistance()) {
                                if (timeX < 1000L) {
                                    durationReal += timeX;
                                } else {
                                    durationReal += 240;
                                }
                                detourLikeNow.setDuration(durationReal);
                                mapStateLike.put(keyLike, detourLikeNow);
                                detourNow.setDuration(durationReal);
                                mapState.put(key, detourNow);
                                log.info("LOG_EVENT_DETOUR  ==>> " + key + "  " + keyLike + "  " + detourNow + "  " + durationReal);
                            }
                        }
                    } else {
                        //计算停驶车辆是否在行驶车辆前
                        double angle = PtInPolyUtil.getAngle(lngR, latR, lngS, latS);
                        boolean isOk = isSyntropy(angle, driveAngleR, 60.0);
                        //这里取距离8米以内的车辆做绕行计算 且
                        double abnormalParkDetourDistanceBeforeStopCar = configModel.getAbnormalParkDetourDistanceBeforeStopCar();
                        if (distance < abnormalParkDetourDistanceBeforeStopCar && isOk) {
                            mapState.put(key, detourNow);
                            mapStateLike.put(keyLike, detourLikeNow);
                        }
                    }
                }
            }
        }
    }


    /**
     * 获取唯一key
     *
     * @param carStop
     * @param carRun
     * @return
     */
    private static String getKey(AccidentFrameModel.CarTrack carStop, AccidentFrameModel.CarTrack carRun) {
        return carStop.getRoadnet().getCrossId() + Constant.MARK + carStop.getId() + Constant.MARK + carRun.getId();
    }

    /**
     * 获取模糊搜索key
     *
     * @param carStop
     * @return
     */
    private static String getKeyLike(AccidentFrameModel.CarTrack carStop) {
        return carStop.getRoadnet().getCrossId() + Constant.MARK + carStop.getId();
    }

    /**
     * 计算行车角度是否同向
     *
     * @param angleA
     * @param angleB
     * @return
     */
    private static boolean isSyntropy(double angleA, double angleB, double checkAngle) {
        double angleX = Math.abs(angleA - angleB);
        if (angleX > 180.0) {
            angleX = 360.0 - angleX;
        }
        if (angleX <= checkAngle) {
            return true;
        } else {
            return false;
        }
    }

}
