package com.wanji.indicators.event.cross.accident.funcnew;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.event.accident.AccidentFrameModel;
import com.wanji.indicators.util.PtInPolyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;

import java.util.*;


/**
 * @auther fengyi
 */
@Slf4j
public class AccidentStopCheckKeyedProcess extends KeyedProcessFunction<String, AccidentFrameModel, AccidentFrameModel> {

    private MapState<String, AccidentFrameModel.CarTrack> mapState;
    private MapState<String, AccidentFrameModel.CarTrack> mapStateStop;

    @Override
    public void open(Configuration parameters) {
        mapState = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateAccident_stop_1", String.class, AccidentFrameModel.CarTrack.class));
        mapStateStop = getRuntimeContext().getMapState(new MapStateDescriptor<>("mapStateAccident_stop_2", String.class, AccidentFrameModel.CarTrack.class));
    }

    @Override
    public void processElement(AccidentFrameModel accidentFrameModel, Context context, Collector<AccidentFrameModel> collector) throws Exception {

        if (accidentFrameModel.isOverFlow() || accidentFrameModel.isDeadLock()) {
            //溢出拥堵或死锁时清除缓存，下次重新开始计算
            mapState.clear();
            mapStateStop.clear();
            accidentFrameModel.getTrackList().clear();
        } else {
            String key = null;
            ConfigModel configModel = accidentFrameModel.getConfigModel();
            long nowTime = accidentFrameModel.getTime();
            Set<String> nowKeys = new HashSet<>();
            List<AccidentFrameModel.CarTrack> list = new ArrayList<>();
            List<AccidentFrameModel.CarTrack> trackList = accidentFrameModel.getTrackList();
            for (AccidentFrameModel.CarTrack carTrack : trackList) {
                key = getKey(carTrack);
                double speed = carTrack.getSpeed();
//                if (carTrack.getGlobalTimeStamp()>=1697265909000L && "鲁AD66123".equals(carTrack.getPicLicense())){
//                    System.out.println();
//                }
                AccidentFrameModel.CarTrack stopPoint = mapStateStop.get(key);
                if (stopPoint != null) {
                    AccidentFrameModel.CarTrack carTrackBefore = mapState.get(key);
                    //当前帧的时间必须在停车时间之后 否则 不予计算 (过滤乱序数据)
                    if (nowTime > stopPoint.getGlobalTimeStamp() && nowTime > carTrackBefore.getGlobalTimeStamp()) {
                        //计算上一帧数据和当前数据时间差
                        long frameTimeX = nowTime - carTrackBefore.getGlobalTimeStamp();
                        if (frameTimeX > 0 && frameTimeX < 1000L) {
                            //计算停车持续时间，单位毫秒
                            long stopDuration = nowTime - stopPoint.getGlobalTimeStamp();
                            carTrack.setStopDuration(stopDuration);
                            //大于此停车车速阈值(外部配置文件设置)的车辆且位移一定距离后移除对其的跟踪，【过滤停车抖动车】
                            if (speed > configModel.getAbnormalParkStopShakeSpeed()) {
                                double distance = PtInPolyUtil.getDistance(carTrack.getLongitude(), carTrack.getLatitude(), stopPoint.getLongitude(), stopPoint.getLatitude());
                                //位移超过 5.0m 且车速超过 3.6km/h 的才算正常行驶
                                if (distance > configModel.getAbnormalParkStopShakeRange()) {
                                    mapStateStop.remove(key);
                                    //log.info( "distance 清除移动车辆 - distance: " + distance + "  speed:" + speed + " 阈值:" + ConfigUtil.getInstance().getConfig().getStopShakeSpeed());
                                }
                            }
                            if (speed > configModel.getAbnormalParkStopRemoveSpeed()) {
                                //车速超过 9km/h 的算正常行驶
                                mapStateStop.remove(key);
//                                            log.info( "speed 清除正常车速车辆 - key: " + speed);
                            }
                            if (carTrack.isHaveConflictCar()) {
                                //剔除因冲突方向车流导致的停车
                                mapStateStop.remove(key);
                                log.info( "剔除因冲突方向车流导致的停车: " +carTrack.getPicLicense()+"@"+ carTrack.getRoadnet().toString());
                            }
                        }
                    }
                } else if (stopPoint == null && speed <= configModel.getAbnormalParkStopSpeed()) {
                    //判定车辆停驶的车速阈值  车辆小于等于此阈值时认为车辆停驶，
                    carTrack.setStop(true);
                    mapStateStop.put(key, carTrack);
                }
                nowKeys.add(key);
                mapState.put(key, carTrack);

                //因为后面算子要判定停止线前车辆运行状态，修改此处（list.add）放到外层
                list.add(carTrack);
            }
            //清除过期数据
            List<AccidentFrameModel.CarTrack> listNowFrame = new ArrayList<>();
            Iterator<AccidentFrameModel.CarTrack> iterator = mapState.values().iterator();
            while (iterator.hasNext()) {
                listNowFrame.add(iterator.next());
            }
            for (AccidentFrameModel.CarTrack next : listNowFrame) {
                key = getKey(next);
                if (!nowKeys.contains(key)) {
                    long timeX = nowTime - next.getGlobalTimeStamp();
                    //计算此key消失的时间  超过1.7秒的做删除处理  轨迹消失后超时移除状态的时间
                    if (timeX > configModel.getAbnormalParkStopDissipateRemoveTime() && timeX < 3600000L) {
//                    log.info( " 清除消失车辆 - keyRemove: " + keyRemove + " 超时: " + timeX);
                        mapState.remove(key);
                        mapStateStop.remove(key);
                    }
                }
            }

            //发送数据
            if (!list.isEmpty()) {
                for (AccidentFrameModel.CarTrack carTrack : list) {
                    key = getKey(carTrack);
                    AccidentFrameModel.CarTrack carTrackStop = mapStateStop.get(key);
                    if (carTrackStop != null) {
                        carTrack.setStop(true);
                        carTrack.setStopDuration(nowTime - carTrackStop.getGlobalTimeStamp());
                        carTrack.setStopTime(carTrackStop.getGlobalTimeStamp());
                        carTrack.setGlobalTimeStamp(nowTime);
                    } else {
                        carTrack.setStop(false);
                        carTrack.setStopDuration(0);
                        carTrack.setEmergencyBraking(false);
                    }
                }
                accidentFrameModel.setTrackList(list);
                collector.collect(accidentFrameModel);
            }
        }
    }

    /**
     * 获取key
     *
     * @param carTrack
     * @return
     */
    private static String getKey(AccidentFrameModel.CarTrack carTrack) {
        return carTrack.getRoadnet().getCrossId() + Constant.MARK + carTrack.getId();
    }


}
