package com.wanji.indicators.event.cross.conflictpoint.func;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.EventAbnormalEnum;
import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.model.ConflictPointModelBase;
import com.wanji.indicators.model.event.confilct.EventCross;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.util.Collector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ConflictEncapsulationResultFlatMap implements FlatMapFunction<EventCross, String> {
    private static final Logger log = LoggerFactory.getLogger(ConflictEncapsulationResultFlatMap.class);

    @Override
    public void flatMap(EventCross event, Collector<String> out) throws Exception {
        log.info("冲突点日志:  " + event.getTime() + "  " + event.getPlateNumber() + "  " +
                event.getAreaDist() + "  " +
                event.getCrossing() + "  " +
                event.getSpeed() + "  ");

//        if(StringUtils.isNotBlank(event.getPlateNumber())){
        ConflictPointModelBase model = new ConflictPointModelBase();

        model.setCrossId(event.getCrossId());
        model.setTrackId(event.getTrackId());
        model.setPlateNumber(event.getPlateNumber());
        model.setPlateColor(event.getPlateColor());
        model.setPlateType(event.getPlateType());
        model.setVehicleColor(event.getVehicleColor());
        model.setVehicleBrand(event.getVehicleBrand());
        model.setVehicleType(event.getVehicleType() + "");
        model.setCategory(event.getCategory());

        model.setTrackIdNear(event.getTrackIdNear());

        EventCross nearEventCross = event.getNearTrackObj();
        nearEventCross.setConfigModel(null);
        model.setNearTackObj(nearEventCross);

        model.setDuration(event.getDuration());

        model.setEventClass(EventAbnormalEnum.CONFLICT_POINT.getDesc());
        model.setEventType(EventAbnormalEnum.CONFLICT_POINT.getType());
        model.setTypeCode(EventAbnormalEnum.CONFLICT_POINT.getType());

        model.setDesc("路口面冲突点: 车辆 " + event.getPlateNumber() + "行驶方向为 " + PtInPolyUtil.getDir(event.getDriveAngle()));
        model.setDescEn("Intersection conflict point: the driving direction of vehicle " + "行驶方向为 " + PtInPolyUtil.getDirEn(event.getDriveAngle()));
        model.setSpeed(event.getSpeed());
        model.setPictureTime1(event.getTime() - 3000);
        model.setPictureTime2(event.getTime());
        model.setPictureTime3(event.getTime() + 3000);
        String matchLane = event.getMatchLane();
        String entryLane = event.getEntryLane();

        model.setTime(model.getTime());

        if (StringUtils.isNotEmpty(matchLane)) {
            model.setLaneId(matchLane);
            model.setRid(matchLane.substring(0, 23));
        } else {
            if (StringUtils.isNotEmpty(entryLane)) {
                try {
                    model.setLaneId(entryLane);
                    model.setRid(entryLane.substring(0, 23));
                } catch (Exception e){
                    e.printStackTrace();;
                }

            }
        }
        long time = event.getTime() / 1000 * 1000;
        model.setStartTime(time - 1000L);
        model.setStartTimeStr(DateUtil.toDateTime(model.getStartTime(), DateUtil.YYYY_MM_DD_HH_MM_SS));
        model.setEndTime(time + 1000L);
        model.setEndTimeStr(DateUtil.toDateTime(model.getEndTime(), DateUtil.YYYY_MM_DD_HH_MM_SS));
        model.setX(event.getLng());
        model.setY(event.getLat());
        model.setTime(model.getStartTime());

        model.setRecoveryCross(event.getRecoveryCross());
        model.setId(model.getCrossId() + Constant.MARK + model.getTrackIdNear() + Constant.MARK + model.getEventType() + Constant.MARK + model.getStartTime());

        if (model.getEndTime() > 0L &&
                model.getStartTime() > 0L &&
                model.getEndTime() > model.getStartTime()) {
            out.collect(JSONObject.toJSONString(model));
        }
        model = null;
//        }
    }


}
