package com.wanji.indicators.model.event.accident;


import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.ConfigModel;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import lombok.Data;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
@Data
public class AccidentFrameModel implements Serializable {

    private String accidentType;
    private String crossId;
    private String rid;
    private long time;
    private List<AccidentFrameModel.CarTrack> trackList;
    private Map<String,Integer> congestionInfo;
    private ConfigModel configModel = new ConfigModel();
    //路口各进口道灯态数据
    private List<CrossRidTurnLampStatusModel> ridTurnLampList = new ArrayList<>();
    //冲突相位信息
    private Map<String, List<String>> conflictSlotMap = new HashMap<>();
    //死锁状态
    private boolean isDeadLock;

    //溢出状态
    private boolean isOverFlow;
    //路口信号周期
    private int cycle;

    public AccidentFrameModel() {
    }

    @Data
    public static class CarTrack extends CarTrackModel implements Serializable {
        /**
         * 辅助计算字段
         */
        //当前帧是否停车 true停车  false行驶
        private boolean isStop;
        //停车持续时间 单位毫秒
        private long stopDuration;
        //停车开始时间 时间戳
        private long stopTime;
        //车辆绕行累计持续时间
        private long detourDuration;
        //车辆绕行累计达到2次后停车持续时间
        private long detourStopDuration;
        //绕行车辆数
        private int detourNum;
        //待转区长度 >> 待转区停止线距离入口车道停止线的距离
        private int waitLaneDistance;
        //是否拥堵
        private boolean isCongestion;
        //拥堵开始时间
        private long congestionStartTime;
        //拥堵结束时间
        private long congestionEndTime;
        //拥堵区域出口rid
        private String congestionOutRid;
        //是否紧急制动
        private boolean isEmergencyBraking;
        //紧急制动最大加速度
        private double emergencyBrakingAt;
        //紧急制动车速
        private double emergencyBrakingSpeed;
        //ttc碰撞预警
        private boolean isTTC;
        //ttc碰撞预警时的车速
        private double ttcSpeed;
        //ttc检测到的相交车辆trackId
        private int ttcTrackId;
        //事故车辆组合后匹配的最近距离trackId
        private int accidentTrackId;
        //事故车辆组合后与其他车辆的距离
        private double accidentDis;
        //停车轨迹稳定持续时间
        private long stopStableDuration;
        //停车时间阈值
        private long checkTime;
        //是否绿灯停车
        private boolean isGreenStop;
        //绿灯停车持续时间
        private long greenStopDuration;
        //绿灯停车开始时间
        private long greenStopStartTime;
        //分组key  = crossId + trackId
        private String key;
        //key:frid value:右转出口道可进入的其他方向进口道，格式：rid;turn_dir_no
        private Map<String,String> ridTurnRefMap = new HashMap<>();
        //三色预警蓝色预警开关
        private String isOpenBlue;

        /********************************************/
        //是否为直左放行相位
        private boolean isStraightLeftThrough;
        //无冲突车辆绿灯持续时间
        private long noCflictCarStopDuration;
        //有冲突车流绿灯内停车次数
        private int hasCflictCarStopCount;
        //有冲突车流绿灯内停车时长
        private long hasCflictCarStopDuration;
        //下个相位绿灯停车时长
        private long nextPhaseStopDuration;
        //下个相位绿灯停车次数
        private long nextPhaseStopCount;
        //是否为下一个相位
        private  boolean isNextPhase;
        //是否满足直、左对方异常停车输出条件
        boolean isSatisfyStraightLeftEvent;
        //标记是否为无灯态正常停车
        private boolean isNormalStopOfNoLampstate;
        //标记是停车车辆是否有同转向车道车辆
        private boolean isHasSameTurnCar;
        //冲突方向是否有车流行驶
        private boolean isHaveConflictCar;
        /********************************************/

    }
}
