package com.wanji.indicators.service.impl;

import com.vividsolutions.jts.geom.*;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.entity.BaseRidInfo;
import com.wanji.indicators.mapper.BaseLaneInfoMapper;
import com.wanji.indicators.service.BaseLaneInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.wanji.indicators.util.CommonUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import com.wanji.indicators.util.Tools;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 车道基础信息 服务实现类
 * </p>
 *
 * @author fengyi
 * @since 2023-04-06
 */
@Service
public class BaseLaneInfoServiceImpl extends ServiceImpl<BaseLaneInfoMapper, BaseLaneInfo> implements BaseLaneInfoService {

    @Resource
    private BaseLaneInfoMapper baseLaneInfoMapper;

    @Override
    public List<BaseLaneInfo> findLaneRidInfo() {
        List<BaseLaneInfo> list = null;//baseLaneInfoMapper.findAllBaseLaneInfo();
        for (BaseLaneInfo model : list) {
            String wkt = model.getRidWkt();
            LineString lineString = GeomsConvertUtil.getLineString(wkt);
            double ridLength = lineString.getLength();
            model.setRidLength(ridLength);
        }
        return list;
    }

    @Override
    public Map<String, BaseLaneInfo> findLaneInfo(String crossList) {
        String[] arr = StringUtils.split(crossList, ",");
        List<String> crosslist = Arrays.asList(arr);
        Map<String, Object> params = new HashMap<>();
        params.put("crossList", crosslist);

        List<BaseLaneInfo> list = this.baseLaneInfoMapper.findCrossLaneInfo(params);
        // t.cross_id,t.road_id,t.rid,t.type,t.dir,count(*) lane_num,t.road_wkt

        Map<String, BaseLaneInfo> laneInfoMap = new HashMap<>();
        for (BaseLaneInfo baseLaneInfo : list) {
            String startPoint = baseLaneInfo.getStartPoint();
            String[] p1 = startPoint.split(",");
            String endPoint = baseLaneInfo.getEndPoint();
            String[] p2 = endPoint.split(",");

            double length = PtInPolyUtil.getDistance(Double.parseDouble(p1[0]),Double.parseDouble(p1[1]),Double.parseDouble(p2[0]),Double.parseDouble(p2[1]));
            baseLaneInfo.setLength(length);

            String ridWkt = baseLaneInfo.getRidWkt();
            double ridLength = GeomsConvertUtil.getSegmentTotalLeng(ridWkt);
            baseLaneInfo.setRidLength(ridLength);

            laneInfoMap.put(baseLaneInfo.getLaneId(), baseLaneInfo);
        }
        return laneInfoMap;
    }

    @Override
    public Map<String, List<String>> getRidTurnConflictInfo(String crossList) {
        Map<String, List<String>> conflictTurnMap = new HashMap<>();

        String[] arr = StringUtils.split(crossList, ",");
        List<String> crosslist = Arrays.asList(arr);
        Map<String, Object> params = new HashMap<>();
        params.put("crossList", crosslist);

        List<Map<String, Object>> list = baseLaneInfoMapper.findRidInfo(params);
        //按路口分组
        Map<String, List<Map<String, Object>>> groupList = list.stream().collect(Collectors.groupingBy(o -> o.get("cross_id").toString()));
        for (Map.Entry<String, List<Map<String, Object>>> entry : groupList.entrySet()) {

            if (entry.getKey().equals("13NGH0B5RC0")) {
                System.out.println();
            }
            List<Map<String, Object>> value = entry.getValue();
            //按角度排序
            value = value.stream().sorted(Comparator.comparing(o -> Double.parseDouble(o.get("end_angle").toString()))).collect(Collectors.toList());

            Map<Integer, String> ridDirMap = new TreeMap<>();

            Set<Integer> dirSets = new HashSet<>();

            for (Map<String, Object> dataMap : value) {
                String crossId = Tools.getMapValue("cross_id", dataMap);
                String rid = Tools.getMapValue("rid", dataMap);
                String ridDir = Tools.getMapValue("rid_dir", dataMap);
                String angle = Tools.getMapValue("end_angle", dataMap);
                //计算4方位方向
                int ridDir4 = CommonUtil.getAngleDir(Double.parseDouble(angle));
                if (dirSets.contains(ridDir4)) {
                    ridDir4 = ridDir4 + 1;
                }
                String crid = crossId + Constant.MARK + rid+ Constant.MARK + ridDir;
                ridDirMap.put(ridDir4, crid);
            }

            for (Map.Entry<Integer, String> entry1 : ridDirMap.entrySet()) {
                Integer dir4 = entry1.getKey();
                String crid = entry1.getValue();
                //顺时针方向
                int clockwise = (dir4 + 1) % 4 == 0 ? 4 : (dir4 + 1) % 4;
                //逆时针直行相位
                int antiClockWise = (dir4 - 1) % 4 == 0 ? 4 : (dir4 - 1) % 4;
                //对向
                int across = ((dir4 + 2) % 4 == 0 ? 4 : (dir4 + 2) % 4) ;

                Integer[] turns = new Integer[]{1, 2};
                for (Integer turn : turns) {
                    String key = crid + Constant.MARK + turn;

                    List<String> conflictTurnList = new ArrayList<>();
                    if (turn == 2) {
                        //与顺时针方向直行冲突
                        String clockwiseConflictStraightTurn = ridDirMap.get(clockwise) + Constant.MARK + "2";
                        //与逆时针方向直行冲突
                        String antiConflictStraightTurn = ridDirMap.get(antiClockWise) + Constant.MARK + "2";
                        //与对向左转冲突
                        String acrossConflictLeftTurn = ridDirMap.get(across) + Constant.MARK + "1";
                        //与逆时针左转冲突
                        String antiClockWiseConflictLeftTurn = ridDirMap.get(antiClockWise) + Constant.MARK + "1";

                        conflictTurnList.add(clockwiseConflictStraightTurn);
                        conflictTurnList.add(antiConflictStraightTurn);
                        conflictTurnList.add(acrossConflictLeftTurn);
                        conflictTurnList.add(antiClockWiseConflictLeftTurn);
                    } else if (turn == 1) {
                        //与顺时针方向直行冲突
                        String clockwiseConflictStraightTurn = ridDirMap.get(clockwise) + Constant.MARK + "2";
                        //与顺时针方向左转冲突
                        String antiConflictLeftTurn = ridDirMap.get(clockwise) + Constant.MARK + "1";
                        //与对向直行冲突
                        String acrossConflictStraightTurn = ridDirMap.get(across) + Constant.MARK + "2";
                        //与逆时针左转冲突
                        String antiClockWiseConflictLeftTurn = ridDirMap.get(antiClockWise) + Constant.MARK + "1";

                        conflictTurnList.add(clockwiseConflictStraightTurn);
                        conflictTurnList.add(antiConflictLeftTurn);
                        conflictTurnList.add(acrossConflictStraightTurn);
                        conflictTurnList.add(antiClockWiseConflictLeftTurn);
                    }
                    conflictTurnMap.put(key, conflictTurnList);
                }

            }
        }
        return conflictTurnMap;
    }

    @Override
    public double getAreaFreeSpeed(String crossList, Double defaultFreeSpeed) {
          Map<String, Map<String,Double>> ridFreeSpeed = this.getRidFreeSpeed(crossList, defaultFreeSpeed);
          Collection<Map<String,Double>> collection = ridFreeSpeed.values();
          return collection.stream().mapToDouble(o->o.get("freeSpeed")).summaryStatistics().getAverage();
    }

    @Override
    public Map<String, Map<String,Double>> getRidFreeSpeed(String crossList, Double defaultFreeSpeed) {
        Map<String, Map<String,Double>> retmap = new HashMap<>();
        String[] sps = crossList.split(",");
        Map<String,Object> params = new HashMap<>();
        params.put("crossList",Arrays.asList(sps));
        List<Map<String, Object>> list = baseLaneInfoMapper.findRidFreeSpeed(params);

        for (Map<String, Object> map : list) {
            String crossId = Tools.getMapValue("cross_id", map);
            String rid = Tools.getMapValue("rid", map);
            String fs = Tools.getMapValue("free_speed", map);
            fs =  fs == null ? defaultFreeSpeed.toString() : fs;

            String level = Tools.getMapValue("level",map);
            String wkt = Tools.getMapValue("wkt",map);
            double ridLength = GeomsConvertUtil.getSegmentTotalLeng(wkt);
            double freeSpeed = Double.parseDouble(fs);

            String key =   rid;

            Map<String,Double> dataMap = new HashMap<>();
            dataMap.put("level",Double.parseDouble(level));
            dataMap.put("ridLength",ridLength);
            dataMap.put("freeSpeed",freeSpeed);

            retmap.put(key, dataMap);
        }
        return retmap;
    }
}
