package com.wanji.indicators.task;

import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.source.debug.DebugSourceGlobalFrame;
import com.wanji.indicators.source.debug.DebugSourceGlobalFrameDtFile;
import com.wanji.indicators.task.area.service.AreaIndexAnalysisMainNew;
import com.wanji.indicators.task.area.service.RidParticipantAnalysisMainNew;
import com.wanji.indicators.task.track.service.CarRoutePathMainNew;
import com.wanji.indicators.task.track.service.TrackStoreMainNew;
import com.wanji.indicators.task.track.service.func.CarTrackFlatMap;
import com.wanji.indicators.task.track.service.func.FrameFlatMap;
import com.wanji.indicators.task.trafficstate.RidTrafficStateMain;
import com.wanji.indicators.util.CommonUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.streaming.api.datastream.DataStreamSource;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 离线计算入口  com.mapabc.project.event.cross.stream.OfflineTestMain
 * @author ke.han
 */
public class OfflineTestIndicatorTaskMain {

    private static final Logger log = LoggerFactory.getLogger(OfflineTestIndicatorTaskMain.class);

    public static void main(String[] args) {
        //初始化环境
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        env.setParallelism(1);

        //加载配置项 并获取工作路径
        String path = new CommonUtil().getWorkSpasePath();
        String outPath = "d:/flink/out/";
        String osName = System.getProperties().getProperty("os.name");
        if (StringUtils.isNotBlank(osName) && osName.contains("Windows")) {
            //此处是 Windows 配置
        } else {
            path = "/opt/flink/workspace/";
        }

        //读取离线数据
        DataStreamSource<String> streamSource = env.addSource(new DebugSourceGlobalFrameDtFile(path));

        SingleOutputStreamOperator<FrameModel> frameModelStream =
                streamSource
                        .flatMap(new FrameFlatMap())
                        .setParallelism(1)
                        .name("轨迹帧数据-JsonToObject");

        SingleOutputStreamOperator<CarTrackModel> carTrackModelStream =
                frameModelStream.
                        flatMap(new CarTrackFlatMap())
                        .setParallelism(1)
                        .name("轨迹帧数据解析-ToCarTrackModel");

        //RidTrafficStateMain.init(env,outPath,false).run(carTrackModelStream);
        try {
            //批量轨迹封装
            TrackStoreMainNew.init(env,outPath,true).run(carTrackModelStream);
            //车辆行驶路径分析
            CarRoutePathMainNew.init(env,outPath,true).run(carTrackModelStream);

            env.execute("离线调试测试");
        } catch (Exception e) {
            log.error("事件测试 - 计算任务异常 : " + e);
        }
    }
}
