package com.wanji.indicators.task.area.service;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.sink.JsonStringFileSink;
import com.wanji.indicators.source.AreaFreeSpeedSource;
import com.wanji.indicators.task.area.service.func.RidCarProcessWindow;
import com.wanji.indicators.task.area.service.func.RidFreeSpeedCoFlatMap;
import com.wanji.indicators.task.area.service.func.RidOutProcessWindow;
import com.wanji.indicators.task.area.service.model.ParticipantAnalysisMidResultModel;
import com.wanji.indicators.task.area.service.model.ParticipantAnalysisOutputModel;
import com.wanji.indicators.task.track.service.func.CarTrackFlatMap;
import com.wanji.indicators.util.PropertiesHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.assigners.TumblingProcessingTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaProducer;
import org.apache.flink.streaming.connectors.kafka.internals.KeyedSerializationSchemaWrapper;
import org.apache.flink.util.Collector;
import org.springframework.beans.BeanUtils;

import java.io.Serializable;
import java.time.Duration;
import java.util.Properties;

@Slf4j
public class RidParticipantAnalysisMainNew implements Serializable {
    private final StreamExecutionEnvironment env;

    private final PropertiesHelper instance;

    private Properties p = new Properties();

    private String path;

    private String indexName = "area.realtime.indicator";

    private int windowTime = 15;

    //本地调试设为true输出结果保存到文件
    private boolean isDebug = false;

    private RidParticipantAnalysisMainNew(StreamExecutionEnvironment env, String path, boolean isDebug) {
        this.env = env;
        this.instance = PropertiesHelper.getInstance();
        this.p = instance.getProperties();
        this.path = path;
        this.isDebug = isDebug;
    }
    public static RidParticipantAnalysisMainNew init(StreamExecutionEnvironment env, String path, boolean isDebug) {
        return new RidParticipantAnalysisMainNew(env, path,isDebug);
    }

    public void run(SingleOutputStreamOperator<CarTrackModel> filterStream) {

        try {
            SingleOutputStreamOperator<CarTrackModel> joinLaneStream = filterStream
                    .connect(env.addSource(new AreaFreeSpeedSource()).broadcast())
                    .flatMap(new RidFreeSpeedCoFlatMap())
                    .setParallelism(1)
                    .name("全域轨迹帧数据-车道属性数据绑定");

            SingleOutputStreamOperator<ParticipantAnalysisMidResultModel> groupByPlateStream =
                    joinLaneStream.assignTimestampsAndWatermarks(
                            WatermarkStrategy.<CarTrackModel>forBoundedOutOfOrderness(Duration.ofSeconds(5))
                                    .withTimestampAssigner((event, timestamp) -> event.getGlobalTimeStamp()))
                            .keyBy(o->(o.getRoadnet().getRid()+"_"+o.getCategory()))
                            .window(TumblingEventTimeWindows.of(Time.minutes(this.windowTime)))
                            //.aggregate(new TravelAggregateFunction(),new TravelProcessWindow())
                            .process(new RidCarProcessWindow())
                            .name("行人非机动车对机动车速度影响计算-按RID_参与者类别分组");

            SingleOutputStreamOperator<ParticipantAnalysisOutputModel> areaStream =
                    groupByPlateStream.assignTimestampsAndWatermarks(
                            WatermarkStrategy.<ParticipantAnalysisMidResultModel>forBoundedOutOfOrderness(Duration.ofSeconds(5))
                                    .withTimestampAssigner((event, timestamp) -> event.getTimestamp()))
                            .keyBy(o->(o.getRid()))
                            .window(TumblingEventTimeWindows.of(Time.seconds(10)))
                            //.aggregate(new AreaTravelAggregateFunction(),new AreaTravelProcessWindow())
                            .process(new RidOutProcessWindow())
                            .name("行人非机动车对机动车速度影响计算-按RID分组计算");

            //最后一条记录存储到redis
            //areaStream.addSink( new RedisSink<ParticipantAnalysisOutputModel>(new RedisConfig().getRedisConfig(), new TravelRedisMapper(Constant.RID_REALTIME_INDICATOR+"_10M")));

            SingleOutputStreamOperator<String> toJsonStream = areaStream.flatMap(new FlatMapFunction<ParticipantAnalysisOutputModel, String>() {
                @Override
                public void flatMap(ParticipantAnalysisOutputModel value, Collector<String> out) throws Exception {
                    out.collect(JSONObject.toJSONString(value));
                }
            }).name("行人非机动车对机动车速度影响计算-转换为json");

            if (!isDebug) {
                // 初始化kafka生产者
                FlinkKafkaProducer producer = new FlinkKafkaProducer<>(p.getProperty("rid.participant.analysis.data.topic"),
                        new KeyedSerializationSchemaWrapper<>(new SimpleStringSchema()),
                        instance.getProducerProperties(), FlinkKafkaProducer.Semantic.NONE);
                //存储到kafka
                toJsonStream.addSink(producer).name("行人非机动车对机动车速度影响计算-数据发送至kafka").setParallelism(1);
            }else {
                String path = this.path + "rid.participant.analysis";
                toJsonStream.addSink(new JsonStringFileSink(path)).setParallelism(1).name("行人非机动车对机动车速度影响计算-结果输出至文件");

            }
        } catch (Exception e) {
            log.error("行人非机动车对机动车速度影响计算",e);
        }
    }


}
