package com.wanji.indicators.task.lightstatus;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.streaming.api.functions.co.CoFlatMapFunction;
import org.apache.flink.util.Collector;

import java.util.*;

@Slf4j
public class CrossLightStatusInfoCoFlatMap implements CoFlatMapFunction<CrossFrameModel, String, CrossFrameModel> {

    String lightStatusJson = null;

    @Override
    public void flatMap1(CrossFrameModel value, Collector<CrossFrameModel> out) throws Exception {
        if (lightStatusJson != null) {
            //解析灯态json
            JSONArray jsonArray = JSON.parseArray(lightStatusJson);
            int size = jsonArray.size();

            List<CrossRidTurnLampStatusModel> list = new ArrayList<>();

            for (int i = 0; i < size; i++) {
                JSONObject jsonObject = jsonArray.getJSONObject(i);
                String crossId = jsonObject.getString("crossId");
                long globalTime = value.getGlobalTimeStamp()  ;
                long lightStatusTime = jsonObject.getLong("timeStamp")  ;
                long difTime = globalTime - lightStatusTime;
                if (globalTime != lightStatusTime) {
                    log.info("轨迹时间：{}，信号时间：{}", DateUtil.toDateTime(globalTime,DateUtil.YYYY_MM_DD_HH_MM_SS), DateUtil.toDateTime(lightStatusTime,DateUtil.YYYY_MM_DD_HH_MM_SS));
                }
                if (!Objects.equals(value.getCrossId(), crossId)) {
                    continue;
                }

                Integer cycleCountDown = jsonObject.getInteger("cycleCountDown");
                Integer cyclePhaseCountDown = jsonObject.getInteger("cyclePhaseCountDown");
                Integer cycleLen = jsonObject.getInteger("cycleLen");

                JSONObject jsonObject1 = jsonObject.getJSONObject("dirLampGroupMap");
                //遍历8方位方向
                for (int j = 1; j <= 8; j++) {
                    JSONObject jsonObject2 = jsonObject1.getJSONObject(String.valueOf(i));
                    if (jsonObject2 != null) {
                        Set<String> keys = jsonObject2.keySet();
                        for (String key : keys) {
                            String status = jsonObject2.getString(key);
                            //灯态信息
                            CrossRidTurnLampStatusModel statusModel = new CrossRidTurnLampStatusModel();
                            statusModel.setCrossId(crossId);
                            statusModel.setDir(i);
                            statusModel.setCycleCountDown(cycleCountDown);
                            statusModel.setCycleLen(cycleLen);
                            statusModel.setCyclePhaseCountDown(cyclePhaseCountDown);
                            statusModel.setLampState(status);
                            statusModel.setTurn(key);
                            list.add(statusModel);
                        }
                    }
                }

            }
            //绑定灯态
            value.setRidTurnLampList(list);
            out.collect(value);
        }
    }

    @Override
    public void flatMap2(String value, Collector<CrossFrameModel> out) throws Exception {
        lightStatusJson = value;
    }
}
