package com.wanji.indicators.task.pedestrian.service.func;

import com.vividsolutions.jts.geom.Geometry;
import com.wanji.indicators.cache.PedestrianDataCache;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.constant.LightStatusEnum;
import com.wanji.indicators.constant.TurnEnum;
import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.task.lightstatus.model.VehicleHeadTimeModel;
import com.wanji.indicators.task.pedestrian.service.model.PedCrossingWalkIndicatorResult;
import com.wanji.indicators.task.pedestrian.service.model.PedCrossingWalkModel;
import com.wanji.indicators.util.ArithOfBigDecmial;
import com.wanji.indicators.util.DateUtil;
import com.wanji.indicators.util.GeomsConvertUtil;
import com.wanji.indicators.util.PtInPolyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.common.state.StateTtlConfig;
import org.apache.flink.api.common.time.Time;
import org.apache.flink.api.common.typeinfo.BasicTypeInfo;
import org.apache.flink.api.java.typeutils.ListTypeInfo;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.springframework.beans.BeanUtils;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author fengyi
 * @date 2023/3/12
 * @description
 */
@Slf4j
public class PedIndicationProcessFunction extends KeyedProcessFunction<String, CrossFrameModel, PedCrossingWalkIndicatorResult> {

    //车道转向绿灯剩余时长缓存
    private  transient MapState<String, Integer> greenRemainTimeState;
    //车道转向绿灯剩余时长缓存
    private  transient MapState<String, String> lightStatusState;
    //绿灯总时长
    private  transient MapState<String, Integer> greenTotalTimeState;
    //绿灯期间各方向上斑马线内人数
    private   MapState<String, List<PedCrossingWalkModel>> bmxAreaResultState;
    //各方向上等待区内的行人目标
    private transient  MapState<String, List<PedCrossingWalkModel>> waitingAreaResultState;

    @Override
    public void open(Configuration parameters) {
        StateTtlConfig ttlConfig = StateTtlConfig.newBuilder(Time.hours(12))//状态过期时间
                .setUpdateType(StateTtlConfig.UpdateType.OnReadAndWrite)
                .cleanupFullSnapshot()
                .setStateVisibility(StateTtlConfig.StateVisibility.ReturnExpiredIfNotCleanedUp)//不返回过期值
                .build();

        greenRemainTimeState = getRuntimeContext().getMapState(new MapStateDescriptor<>("greenRemainTimeState", String.class, Integer.class));
        bmxAreaResultState = getRuntimeContext().getMapState(new MapStateDescriptor<>("bmxAreaResultState", BasicTypeInfo.STRING_TYPE_INFO, new ListTypeInfo<>(PedCrossingWalkModel.class)));
        greenTotalTimeState = getRuntimeContext().getMapState(new MapStateDescriptor<>("greenTotalTimeState", String.class, Integer.class));
        waitingAreaResultState = getRuntimeContext().getMapState(new MapStateDescriptor<>("waitingAreaResultState", BasicTypeInfo.STRING_TYPE_INFO, new ListTypeInfo<>(PedCrossingWalkModel.class)));
        lightStatusState = getRuntimeContext().getMapState(new MapStateDescriptor<>("lightStatusState", String.class, String.class));

    }

    @Override
    public void processElement(CrossFrameModel value, Context ctx, Collector<PedCrossingWalkIndicatorResult> out) throws Exception {

        long globalTimeStamp = value.getGlobalTimeStamp();

        List<CarTrackModel> trackList = value.getTrackList();
        //筛选行人目标
        trackList = trackList.stream().filter(o -> Objects.nonNull(o.getRoadnet().getRid()) && VehicleTypeEnum.isPedestrian(o.getOriginalType().shortValue())).collect(Collectors.toList());
        //按方向进行分组
        Map<String, List<CarTrackModel>> groupTrackByDir = trackList.stream().collect(Collectors.groupingBy(o -> o.getRoadnet().getRid()));
        //检测等待区行人
        this.checkWaitingAreaObjectList(trackList, globalTimeStamp);

        //灯态数据
        List<CrossRidTurnLampStatusModel> ridLightStatusList = value.getRidTurnLampList();
        //筛选行人绿灯
        Map<String, List<CrossRidTurnLampStatusModel>> groupLightStatus =
                ridLightStatusList.stream()
                        .filter(o -> Objects.nonNull(o.getTurn())
                                && Objects.nonNull(o.getLampState())
                                && Objects.nonNull(o.getRid())
                                // && o.getLampState().equals(LightStatusEnum.GREEN.getType())
                                && o.getTurn().equals("20")
                                // && o.getRid().equals("13NI00B5RM013NGH0B5RC00")
                        )
                        .collect(Collectors.groupingBy(o -> o.getRid()));

        //遍历行人绿灯情况下，斑马线上的行人情况
        for (Map.Entry<String, List<CrossRidTurnLampStatusModel>> lightEntry : groupLightStatus.entrySet()) {
            String rid = lightEntry.getKey();
//            if (rid.equals("13NI00B5RM013NGH0B5RC00")) {
//                System.out.println();
//            }
            List<CrossRidTurnLampStatusModel> value1 = lightEntry.getValue();
            //绿灯放行出现的行人目标
            List<CarTrackModel> runList = groupTrackByDir.get(rid);
            Integer agoGreenTime = greenRemainTimeState.get(rid);
            boolean flag = false;
            if (agoGreenTime == null) {
                //只在第一次进入时缓存作为绿灯时长
                greenTotalTimeState.put(rid, value1.get(0).getCyclePhaseCountDown());
                flag = true;
            }
            // =========================================================================================================//
            //当前灯态
            String nowLightColor = value1.get(0).getLampState();
            String agoLightColor = lightStatusState.get(rid);
            int lightDir = value1.get(0).getDir();
            String turn = value1.get(0).getTurn();
            //当前灯态行人是否可通行
            boolean isCanWalk = false;
            if (Objects.equals(nowLightColor, LightStatusEnum.GREEN.getType()) || nowLightColor.equals(LightStatusEnum.YELLOW.getType())) {
                isCanWalk = true;
            } else if (agoLightColor != null) {
                 if (nowLightColor.equals(LightStatusEnum.RED.getType()) && agoLightColor.equals(LightStatusEnum.YELLOW.getType())) {
                    if (value1.get(0).getCyclePhaseCountDown() <= 5) {
                        //由黄灯变为红灯，倒计时未结束【全红时】
                        isCanWalk = true;
                    }
                }
            }
            // =========================================================================================================//
            // 相位时长倒计时
            greenRemainTimeState.put(rid, value1.get(0).getCyclePhaseCountDown());
            lightStatusState.put(rid, value1.get(0).getLampState());

            List<PedCrossingWalkModel> agoMidResult = bmxAreaResultState.get(rid);
            if (agoMidResult == null) {
                agoMidResult = new ArrayList<>();
            }

            if (isCanWalk) {
//                log.info("行人绿灯长：rid={},行人绿灯剩余时长={}s,行人绿灯总时长={}s，时间：{},当前灯态：{}，上次灯态：{}，方向：{}",
//                        rid, greenRemainTimeState.get(rid), greenTotalTimeState.get(rid), DateUtil.toDateTime(globalTimeStamp, "yyyy-MM-dd HH:mm:ss.SSS"),nowLightColor,agoLightColor,lightDir);

                if (runList != null && !runList.isEmpty()) {
                    //按rid分组
                    Map<String, List<CarTrackModel>> groupByRid = runList.stream().collect(Collectors.groupingBy(o -> o.getRoadnet().getRid()));
                    for (Map.Entry<String, List<CarTrackModel>> entry1 : groupByRid.entrySet()) {
                        String key = entry1.getKey();
                        List<CarTrackModel> pedList = entry1.getValue();
                        //在斑马线区域内的行人
                        List<CarTrackModel> inBmxAreaPedList = new ArrayList<>();
                        for (CarTrackModel pedModel : pedList) {
                            double lng = pedModel.getLongitude();
                            double lat = pedModel.getLatitude();
                            String bmxGeom = PedestrianDataCache.getInstance().getCrossingWalkMap().get(key);
                            if (bmxGeom != null) {
                                //是否在斑马线区域内
                                boolean isInBmxArea = GeomsConvertUtil.withinGeo(lng + "," + lat, bmxGeom);
                                if (isInBmxArea)
                                    inBmxAreaPedList.add(pedModel);
                            }
                        }

                        //监测新帧数据中行人
                        List<PedCrossingWalkModel> newList = new ArrayList<>();
                        for (CarTrackModel nowModel : inBmxAreaPedList) {
                            Integer nowId = nowModel.getId();
                            boolean isNew = true;
                            for (PedCrossingWalkModel agoModel : agoMidResult) {
                                Integer agoId = agoModel.getObjectId();
                                //同一个目标仍在斑马线内更新实时时间戳
                                if (Objects.equals(nowId, agoId)) {
                                    agoModel.setNowTime(globalTimeStamp);
                                    //偏离区域后又进入到区域，重置离开标记
                                    if (agoModel.isLeave()) {
                                        agoModel.setLeave(false);
                                        agoModel.setLeaveTime(0);
                                    }
                                    isNew = false;

                                    double speedDiff = nowModel.getSpeed() - agoModel.getSpeed();
                                    //速度差大于0.5m/s，转换为km/h
                                    if (speedDiff >= 0.5 * 3.6) {
                                        agoModel.setAcceleratedPassDuration(globalTimeStamp - agoModel.getAcceleratedPassStartTime());
                                        agoModel.setAcceleratedPassStartTime(globalTimeStamp);
                                        //持续时长大于5秒，设置为加速通过标记
                                        if (agoModel.getAcceleratedPassDuration() > 5 * 1000) {
                                            agoModel.setAcceleratedPass(true);
                                        }
                                    }

                                }
                            }
                            //斑马线内出现的新目标
                            if (isNew) {
                                newList.add(toPedInfo(nowModel));
                            }
                        }
                        //监测行人是否离开斑马线区域
                        for (PedCrossingWalkModel agoModel : agoMidResult) {
                            Integer agoId = agoModel.getObjectId();
                            boolean matched = inBmxAreaPedList.stream().anyMatch(o -> o.getId().equals(agoId));
                            if (!agoModel.isLeave() && !matched) {
                                //第一次离开区域的时间
                                agoModel.setLeaveTime(globalTimeStamp);
                                agoModel.setLeave(true);
                            }
                        }
                        agoMidResult.addAll(newList);

                        if (flag) {
                            //记录第一次绿灯时进入斑马线区域的对象
                            for (CarTrackModel pedModel : inBmxAreaPedList) {
                                agoMidResult.add(toPedInfo(pedModel));
                            }
                            bmxAreaResultState.put(key, agoMidResult);
                        } else {
                            bmxAreaResultState.put(key, agoMidResult);
                        }

                    }

                }
            } else {//绿灯结束时，输出计算结果
                if (!agoMidResult.isEmpty()) {
                    PedCrossingWalkIndicatorResult result = this.getOutResult(value.getCrossId(), rid, globalTimeStamp);
                    out.collect(result);
                }
//                if (agoGreenTime != null && agoGreenTime==1) {
//                    greenTotalTimeState.remove(rid);
//                }
                greenTotalTimeState.remove(rid);
                bmxAreaResultState.remove(rid);
                lightStatusState.remove(rid);
                greenRemainTimeState.remove(rid);

            }
        }

        Iterator<Map.Entry<String, List<PedCrossingWalkModel>>> waitIterator = waitingAreaResultState.iterator();
        //过期数据清理
        while (waitIterator.hasNext()) {
            Map.Entry<String, List<PedCrossingWalkModel>> entry = waitIterator.next();
            List<PedCrossingWalkModel> list = entry.getValue();
            List<PedCrossingWalkModel> rmList = entry.getValue();
            for (PedCrossingWalkModel model : list) {
                long time = model.getNowTime();
                long difTime = System.currentTimeMillis() - time;
                if (difTime > 5 * 60 * 1000) {
                    rmList.add(model);
                }
            }
            if (!rmList.isEmpty()) {
               // log.info("安全岛过期行人目标清理：rid={}，总人数：{},清理人数：{}", entry.getKey(), list.size(), rmList.size());
            }
            list.removeAll(rmList);
            entry.setValue(list);
        }
    }

    private PedCrossingWalkModel toPedInfo(CarTrackModel pedModel) {
        PedCrossingWalkModel pedCrossingWalkModel = new PedCrossingWalkModel();
        pedCrossingWalkModel.setObjectId(pedModel.getId());
        pedCrossingWalkModel.setArriveTime(pedModel.getGlobalTimeStamp());
        pedCrossingWalkModel.setDir(pedModel.getRoadnet().getRidDir8());
        pedCrossingWalkModel.setRid(pedModel.getRoadnet().getRid());
        pedCrossingWalkModel.setCrossId(pedModel.getRoadnet().getCrossId());
        pedCrossingWalkModel.setNowTime(pedModel.getGlobalTimeStamp());
        pedCrossingWalkModel.setSpeed(pedModel.getSpeed());
        pedCrossingWalkModel.setArriveSpeed(pedModel.getSpeed());
        return pedCrossingWalkModel;
    }

    /**
     * 封装路口指标结果
     *
     * @param crossId
     * @param rid
     * @param globalTimeStamp
     * @return
     */
    private PedCrossingWalkIndicatorResult getOutResult(String crossId, String rid, Long globalTimeStamp) throws Exception {
        //绿灯时间内路口总人数
        int size = 0;
        //离开区域的人数
        long outSize = 0;
        //离开区域的目标总通过时间
        double totalPassTime = 0;
        //加速通过斑马线人数
        long accelerateCount = 0;

        List<PedCrossingWalkModel> waitingList = new ArrayList<>();
        Iterator<Map.Entry<String, List<PedCrossingWalkModel>>> waitIterator = waitingAreaResultState.iterator();
        while (waitIterator.hasNext()) {
            Map.Entry<String, List<PedCrossingWalkModel>> entry = waitIterator.next();
            waitingList.addAll(entry.getValue());
        }
        //各方向斑马线上行人目标
        Iterator<Map.Entry<String, List<PedCrossingWalkModel>>> iterator = bmxAreaResultState.iterator();
        Set<String> ridKeys = new HashSet<>();
        List<PedCrossingWalkModel> walkingList = new ArrayList<>();
        while (iterator.hasNext()) {
            Map.Entry<String, List<PedCrossingWalkModel>> entry = iterator.next();
            String key = entry.getKey();
            List<PedCrossingWalkModel> ridData = entry.getValue();
            size += ridData.size();
            outSize += ridData.stream().filter(o -> o.isLeave()).count();
            totalPassTime += ridData.stream().filter(o -> o.isLeave()).mapToDouble(o -> o.getLeaveTime() - o.getArriveTime()).sum();
            accelerateCount += ridData.stream().filter(o -> o.isAcceleratedPass()).count();

            ridKeys.add(key);
            walkingList.addAll(ridData);
        }

        long totalWaitTime = 0;
        Set<Integer> idSets = new HashSet<>();
        for (PedCrossingWalkModel walkModel : walkingList) {
            Integer id = walkModel.getObjectId();
            Optional<PedCrossingWalkModel> waitOption = waitingList.stream().filter(o -> o.getObjectId().equals(id)).findAny();
            if (waitOption.isPresent()) {
                //第一次进入斑马线时间 - 第一次进入等待区时间
                totalWaitTime += walkModel.getArriveTime() - waitOption.get().getArriveTime();
            }
            idSets.add(id);
        }
        //平均等待时长
        double avgWaitTime = totalWaitTime / walkingList.size();
        if (avgWaitTime < 0){
            avgWaitTime = 0;
        }
        //平均等待人数
        long waitingCount = waitingList.stream().filter(o -> o.isWaiting()).count();

        //绿灯开始时间
        long startTime = globalTimeStamp - greenTotalTimeState.get(rid) * 1000;
        //绿灯结束时间
        long endTime = globalTimeStamp;
        //平均通过率
        double avgPassedRate = ArithOfBigDecmial.div(outSize, size);
        //平均通过时长
        double avgPassTime = ArithOfBigDecmial.div(totalPassTime, size);
        //绿灯计算时加速通过率
        double accRate = ArithOfBigDecmial.div(accelerateCount, size);

        PedCrossingWalkIndicatorResult result = new PedCrossingWalkIndicatorResult();
        result.setCrossId(crossId);
        result.setStartTime(startTime);
        result.setStartTimeStr(DateUtil.toDateTime(startTime,DateUtil.YYYY_MM_DD_HH_MM_SS));
        result.setEndTime(endTime);
        result.setEndTimeStr(DateUtil.toDateTime(endTime,DateUtil.YYYY_MM_DD_HH_MM_SS));
        result.setAvgPassedRate(avgPassedRate);
        result.setAvgPassTime(avgPassTime);
        result.setAccRate(accRate);
        result.setPedCount(size);
        result.setWaitingCount(waitingCount);
        result.setAvgWaitTime(avgWaitTime);

        log.info("绿灯结束时指标结果输出：{}", result);

        //绿灯结束，清理缓存状态
//        for (String key : ridKeys) {
//            bmxAreaResultState.remove(key);
//            greenTotalTimeState.remove(key);
//            greenRemainTimeState.remove(key);
//            lightStatusState.remove(key);
//        }

        waitIterator = waitingAreaResultState.iterator();
        //清除等待区已经计算的行人目标
        while (waitIterator.hasNext()) {
            Map.Entry<String, List<PedCrossingWalkModel>> entry = waitIterator.next();
            List<PedCrossingWalkModel> value = entry.getValue();
            value.removeIf(o -> idSets.contains(o.getObjectId()));
            entry.setValue(value);
        }
        return result;
    }

    //匹配等待区行人
    private void checkWaitingAreaObjectList(List<CarTrackModel> trackList, long globalTimeStamp) throws Exception {
        for (CarTrackModel pedModel : trackList) {
            double lng = pedModel.getLongitude();
            double lat = pedModel.getLatitude();
            Map<String, String> waitingAreaGeomMap = PedestrianDataCache.getInstance().getSafetyIslandMap();
            //检测是否在等待区域
            for (Map.Entry<String, String> entry : waitingAreaGeomMap.entrySet()) {
                String areaId = entry.getKey();
                List<PedCrossingWalkModel> agoMidResult = waitingAreaResultState.get(areaId);
                if (agoMidResult == null) {
                    agoMidResult = new ArrayList<>();
                }
                String geom = entry.getValue();
                boolean isInArea = GeomsConvertUtil.withinGeo(lng + "," + lat, geom);
                boolean isNew = true;
                for (PedCrossingWalkModel model : agoMidResult) {
                    Integer id = model.getObjectId();
                    //下一帧还在等待区，更新当前时间
                    if (Objects.equals(id, pedModel.getId())) {
                        if (isInArea) {
                            model.setNowTime(globalTimeStamp);
                            long difTime = globalTimeStamp - model.getArriveTime();
                            if (difTime > 3 * 1000) {
                                //行人等待标记
                                model.setWaiting(true);
                            }
                            model.setWaitingTime(difTime);
                        }
                        isNew = false;
                        break;
                    }
                }
                if (isNew) {
                    agoMidResult.add(toPedInfo(pedModel));
                }
                waitingAreaResultState.put(areaId, agoMidResult);
            }
        }
    }
}
