package com.wanji.indicators.task.pedestrian.service.func;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.cache.BaseLaneCache;
import com.wanji.indicators.cache.CrossLightStatusDataCache;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.entity.BaseLaneInfo;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.util.DateUtil;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.util.Collector;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

public class TrafficLightStatusFlatMap implements FlatMapFunction<String, Map<String, List<CrossRidTurnLampStatusModel>>> {
    @Override
    public void flatMap(String value, Collector<Map<String, List<CrossRidTurnLampStatusModel>>> out) throws Exception {


        Map<String, BaseLaneInfo> laneInfoMap = BaseLaneCache.getInstance().getBaseLaneInfoMap();

        Map<String, List<CrossRidTurnLampStatusModel>> groupMap = new HashMap<>();

        //解析灯态json
        JSONArray jsonArray = JSON.parseArray(value);
        int size = jsonArray.size();
        long lightStatusTime = 0;
        for (int i = 0; i < size; i++) {
            JSONObject jsonObject = jsonArray.getJSONObject(i);
            String crossId = jsonObject.getString("crossId");
            lightStatusTime = jsonObject.getLong("timeStamp");
            Integer cycleCountDown = jsonObject.getInteger("cycleCountDown");
            Integer cyclePhaseCountDown = jsonObject.getInteger("cyclePhaseCountDown");
            Integer cycleLen = jsonObject.getInteger("cycleLen");

            List<BaseLaneInfo> laneList = laneInfoMap.values().stream().filter(o -> o.getCrossId().equals(crossId)).collect(Collectors.toList());

            JSONObject jsonObject1 = jsonObject.getJSONObject("dirLampGroupMap");

            List<CrossRidTurnLampStatusModel> list = groupMap.get(crossId);
            if (list == null) {
                list = new ArrayList<>();
            }
            //遍历8方位方向
            for (int j = 1; j <= 8; j++) {
                //方向灯态数据
                JSONObject jsonObject2 = jsonObject1.getJSONObject(String.valueOf(j));
                if (jsonObject2 != null) {
                    Set<String> keys = jsonObject2.keySet();
                    //key: 1左 2直 3右 20行人
                    for (String key : keys) {
                        String status = jsonObject2.getString(key);
                        AtomicInteger atomicInteger = new AtomicInteger(j);
                        Optional<BaseLaneInfo> optional = laneList.stream().filter(o -> o.getType().equals(2) && o.getDir() == atomicInteger.get()).findAny();
                        //灯态信息
                        CrossRidTurnLampStatusModel statusModel = new CrossRidTurnLampStatusModel();
                        statusModel.setCrossId(crossId);
                        statusModel.setDir(j);
                        if (optional.isPresent()) {
                            statusModel.setRid(optional.get().getRid());
                        }
                        statusModel.setCycleCountDown(cycleCountDown);
                        statusModel.setCycleLen(cycleLen);
                        statusModel.setCyclePhaseCountDown(cyclePhaseCountDown);
                        statusModel.setLampState(status);
                        statusModel.setLightStatusTime(lightStatusTime);
                        statusModel.setTurn(key);
                        list.add(statusModel);
                    }
                }
            }
            groupMap.put(crossId, list);
            //缓存灯态
//            String key = crossId + Constant.MARK + lightStatusTime / 1000;
//            CrossLightStatusDataCache.addCrossTimeLight(crossId, list);
            CrossLightStatusDataCache.getCrossLightStatusMap().put(crossId, list);
        }
        out.collect(groupMap);
    }

}
