package com.wanji.indicators.task.pedestrian.stream;

import com.alibaba.fastjson.JSONObject;
import com.wanji.indicators.event.stream.func.CrossFrameFlatMap;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.CrossRidTurnLampStatusModel;
import com.wanji.indicators.source.LaneInfoSource;
import com.wanji.indicators.source.websocket.LightStatusWebSocketSource;
import com.wanji.indicators.task.pedestrian.service.func.PedIndicationProcessFunction;
import com.wanji.indicators.task.pedestrian.service.func.RefPedRidInfoCoFlatMap;
import com.wanji.indicators.task.pedestrian.service.func.TrafficLightStatusFlatMap;
import com.wanji.indicators.task.pedestrian.service.func.TrafficLightStatusInfoCoFlatMap;
import com.wanji.indicators.task.pedestrian.service.model.PedCrossingWalkIndicatorResult;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FilterFunction;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.runtime.state.storage.FileSystemCheckpointStorage;
import org.apache.flink.streaming.api.datastream.ConnectedStreams;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.DataStreamSource;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.CheckpointConfig;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaProducer;
import org.apache.flink.streaming.connectors.kafka.internals.KeyedSerializationSchemaWrapper;
import org.apache.flink.util.Collector;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.serialization.StringDeserializer;

import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.UUID;

public class PedestrianMain {
    //private static final Logger log = LoggerFactory.getLogger(TrafficEventMain.class);
    private static final String GROUP_ID = "FlinkConsumerGroup-V56-TRACK-PED";

    public static void main(String[] args) {

        PropertiesHelper instance = PropertiesHelper.getInstance();
        Properties properties = instance.getProperties();
        String topic = properties.getProperty("consumer.topic");

        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        env.setParallelism(1);

//        env.enableCheckpointing(10 * 60 * 1000);
//        env.getCheckpointConfig().setTolerableCheckpointFailureNumber(3);
//        env.getCheckpointConfig().setCheckpointTimeout(10 * 60 * 1000);
//        env.getCheckpointConfig().setMaxConcurrentCheckpoints(10);
//        env.getCheckpointConfig().setCheckpointStorage(new FileSystemCheckpointStorage(properties.getProperty("check.point.uri")));
//        env.getCheckpointConfig().setExternalizedCheckpointCleanup(CheckpointConfig.ExternalizedCheckpointCleanup.DELETE_ON_CANCELLATION);
        //============================================================================================//
        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "false")
                .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                .setTopics(topic)
                .setGroupId(GROUP_ID + UUID.randomUUID().toString())
                .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();
        DataStream<String> stream = env
                .fromSource(source, WatermarkStrategy.noWatermarks(), "kafka-car-track-source1");
        //============================================================================================//
        KafkaSource<String> lightStatusSource = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "false")
                .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                .setTopics("cross_lights_status")
                .setGroupId(GROUP_ID + UUID.randomUUID().toString())
                .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))
                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))
                .build();
        DataStream<String> lightStatusStream = env
                .fromSource(lightStatusSource, WatermarkStrategy.noWatermarks(), "cross_lights_status-data-source");

       // DataStreamSource<String> lightStatusStream = env.addSource(new LightStatusWebSocketSource());

        SingleOutputStreamOperator<Map<String,List<CrossRidTurnLampStatusModel>>> turnLightStatusStream =
                lightStatusStream.flatMap(new TrafficLightStatusFlatMap()).setParallelism(1).name("实时灯态信息");

        //============================================================================================//

        SingleOutputStreamOperator<CrossFrameModel> thinningDataStream =
                stream.flatMap(new CrossFrameFlatMap()).setParallelism(1).name("全域轨迹帧数据-按路口分组输出");
        //测试过滤
//        SingleOutputStreamOperator<CrossFrameModel> filterDataStream = thinningDataStream.filter(new FilterFunction<CrossFrameModel>() {
//            @Override
//            public boolean filter(CrossFrameModel value) throws Exception {
//                return value.getCrossId().equals("13NGH0B5RC0");
//            }
//        });

        //连接灯态数据源
        ConnectedStreams<CrossFrameModel, Map<String,List<CrossRidTurnLampStatusModel>>> connStream = thinningDataStream.connect(turnLightStatusStream);

        SingleOutputStreamOperator<CrossFrameModel> bindLightStatusStream =
                connStream.flatMap(new TrafficLightStatusInfoCoFlatMap()).setParallelism(1).name("全域轨迹帧数据-路口灯态绑定");

        SingleOutputStreamOperator<CrossFrameModel> joinLaneStream = bindLightStatusStream
                .flatMap(new RefPedRidInfoCoFlatMap())
                .setParallelism(1)
                .name("全域轨迹帧数据-车道属性数据绑定");

        SingleOutputStreamOperator<PedCrossingWalkIndicatorResult> midResultStream =
                joinLaneStream
                        .keyBy(o -> o.getCrossId())
                        .process(new PedIndicationProcessFunction())
                        .setParallelism(1)
                        .name("行人指标计算");

        SingleOutputStreamOperator<String> toJsonStream=  midResultStream.flatMap(new FlatMapFunction<PedCrossingWalkIndicatorResult, String>() {
            @Override
            public void flatMap(PedCrossingWalkIndicatorResult value, Collector<String> out) throws Exception {
                out.collect(JSONObject.toJSONString(value));
            }
        }).setParallelism(1).name("行人指标转JSON");

        try {

            FlinkKafkaProducer producer = new FlinkKafkaProducer<>("analysis.pedestrian.indicators",
                    new KeyedSerializationSchemaWrapper<>(new SimpleStringSchema()),
                    instance.getProducerProperties(), FlinkKafkaProducer.Semantic.NONE);
            //存储到kafka
            toJsonStream.addSink(producer).name("行人指标计算-数据发送至kafka").setParallelism(1);

            env.execute("行人指标计算");
        } catch (Exception e) {
            e.printStackTrace();
            //log.error("交通指标计算任务异常 :   " + e);
        }
    }
}
