package com.wanji.indicators.task.track.service.func;

import com.wanji.indicators.constant.VehicleTypeEnum;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.model.RoutePathCarTrackModel;
import com.wanji.indicators.util.CarNumGenerator;
import com.wanji.indicators.util.DateUtil;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.util.Collector;
import org.apache.kafka.common.protocol.types.Field;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 帧数据转换
 */
public class CarTrackFlatMap implements FlatMapFunction<FrameModel, CarTrackModel> {

    private static final Logger log = LoggerFactory.getLogger(CarTrackFlatMap.class);
    //孪生出的数据暂时没有车牌号，通过ID映射一个随机车牌号作为测试
    static Map<Integer,String[]> randomCarNumMap = new ConcurrentHashMap<>();
    static  String motorObjectType = "1,2,3,7,8,10,11,12,13,15";
    boolean isMockPlate = false;

    @Override
    public void flatMap(FrameModel frameModel, Collector<CarTrackModel> collector) throws Exception {

        String timeStamp = frameModel.getTimeStamp();
        Long globalTimeStamp = frameModel.getGlobalTimeStamp();
        Integer participantNum = frameModel.getParticipantNum();
        String globalId = frameModel.getOrgCode();

        List<CarTrackModel> list = frameModel.getTrackList();

        if (list != null && !list.isEmpty()) {

            list.forEach(o -> {
                o.setTimeStamp(DateUtil.toDateTime(globalTimeStamp, "yyyy-MM-dd HH:mm:ss.SSS"));
                o.setGlobalTimeStamp(globalTimeStamp);
                o.setOrgCode(globalId);

                short originalType = o.getOriginalType().shortValue();
                int category = 0;
                if (VehicleTypeEnum.isMotorVehicles(originalType)) {
                    category = 1;
                } else if (VehicleTypeEnum.isNonMotorVehicles(originalType)) {
                    category = 2;
                } else {
                    category = 3;
                }
                o.setCategory(category);

                collector.collect(o);
            });
        }
    }

    /**
     * 获取随机数
     *
     * @param min
     * @param max
     * @return
     */
    public static int getRandom(int min, int max) {
        int floor = (int) Math.floor(Math.random() * (max - min + 1) + min);
        return floor;
    }

}
