package com.wanji.indicators.task.track.stream;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.task.track.service.CarRoutePathMainNew;
import com.wanji.indicators.task.track.service.TrackStoreMainNew;
import com.wanji.indicators.task.track.service.func.CarTrackFlatMap;
import com.wanji.indicators.task.track.service.func.FrameFlatMap;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.FilterFunction;
import org.apache.flink.api.java.utils.ParameterTool;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.runtime.state.filesystem.FsStateBackend;
import org.apache.flink.runtime.state.memory.MemoryStateBackend;
import org.apache.flink.runtime.state.storage.FileSystemCheckpointStorage;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.CheckpointConfig;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.TopicPartition;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.joda.time.DateTime;

import java.util.*;

public class TrackAndRouteMain {
    //private static final Logger log = LoggerFactory.getLogger(TrafficEventMain.class);

    public static void main(String[] args) {
        //批量轨迹es index,逗号分割：esIndexName,windowTime
        String indexName = args[0];
        String indexName1 = args[1];

//        ParameterTool parameter = ParameterTool.fromArgs(args);
//        String indexName = parameter.get("Track_ES_IndexName");
//        String indexName1 = parameter.get("Route_ES_IndexName");

        PropertiesHelper instance = PropertiesHelper.getInstance();
        Properties properties = instance.getProperties();

        String topic = properties.getProperty("consumer.topic");
        if (args.length>2){
            topic = args[2];
        }

        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        env.enableCheckpointing(60*1000);
        env.getCheckpointConfig().setTolerableCheckpointFailureNumber(3);
        env.getCheckpointConfig().setCheckpointTimeout(10 * 60 * 1000);
        env.getCheckpointConfig().setCheckpointStorage(new FileSystemCheckpointStorage(properties.getProperty("check.point.uri")));
        //env.setStateBackend(new FsStateBackend(properties.getProperty("check.point.uri")));
        env.getCheckpointConfig().setExternalizedCheckpointCleanup(CheckpointConfig.ExternalizedCheckpointCleanup.DELETE_ON_CANCELLATION);
//        env.setStateBackend(
//                new MemoryStateBackend(MemoryStateBackend.DEFAULT_MAX_STATE_SIZE * 100));


        DateTime currentDateTime = new DateTime();
        DateTime dateTime = currentDateTime.withMillisOfDay(0).minusHours(-4);

        KafkaSource<String> source = KafkaSource.<String>builder()
                .setProperties(instance.getConsumerProperties())
                .setProperty("auto.offset.commit", "true")
                .setProperty("auto.commit.interval.ms", "1000")
                .setProperty("commit.offsets.on.checkpoint", "true")
                .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                .setTopics(topic)
                .setGroupId(properties.getProperty("consumer.group.id"))
                /* 设置起始偏移量有以下几种情况
                       1.从指定的位置消费：OffsetsInitializer.offsets(Map<TopicPartition, Long> offsets)
                       2.从最新位置消费(最后一条处)：OffsetsInitializer.latest()
                       3.从最早位置消费(第一条处)：OffsetsInitializer.earliest()
                       4.从上次提交的位置消费：OffsetsInitializer.committedOffsets()
                       5.新的组，从来没有提交过,再指定一个消费方式：OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST)
                      */
                .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))
                // 从大于等于此时间戳开始的偏移量开始
                //.setStartingOffsets(OffsetsInitializer.timestamp(dateTime.getMillis()))

                .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))

                .build();

        DataStream<String> stream = env
                .fromSource(source, WatermarkStrategy.noWatermarks(), "kafka-car-track-source-route");

        SingleOutputStreamOperator<FrameModel> frameModelStream =
                stream
                        .flatMap(new FrameFlatMap())
                        .setParallelism(1)
                        .name("轨迹帧数据-JsonToObject");

        SingleOutputStreamOperator<CarTrackModel> carTrackModelStream =
                frameModelStream.
                        flatMap(new CarTrackFlatMap())
                        .setParallelism(1)
                        .name("轨迹帧数据解析-ToCarTrackModel");

//        //筛选机动车类型数据
//        SingleOutputStreamOperator<CarTrackModel> filterStream =
//                carTrackModelStream.filter(new FilterFunction<CarTrackModel>() {
//                    @Override
//                    public boolean filter(CarTrackModel value) throws Exception {
//                        String motorObjectType = Constant.MOTOR_TYPES;
//                        String[] sps = motorObjectType.split(",");
//                        List<String> typeList = Arrays.asList(sps);
//                        String carType = value.getOriginalType().toString();
//                        if (typeList.contains(carType)){
//                            return  true;
//                        }
//                        return false;
//                    }
//                }).name("筛选机动车数据");

        try {
            //批量轨迹封装
            TrackStoreMainNew.init(env, indexName).run(carTrackModelStream);

            CarRoutePathMainNew.init(env, indexName1).run(carTrackModelStream);

            env.execute("轨迹处理工作流");

        } catch (Exception e) {
            e.printStackTrace();
            //log.error("交通指标计算任务异常 :   " + e);
        }
    }
}
