package com.wanji.indicators.util;

import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.model.CarTrackModel;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

public class CommonUtil {

    private static final Logger log = LoggerFactory.getLogger(CommonUtil.class);


    public static boolean isInCrossing(CarTrackModel carTrackModel) {
        CarTrackModel.RoadNet roadNet = carTrackModel.getRoadnet();
        String crossId = roadNet.getCrossId();
        String rid = roadNet.getRid();
        String laneId = roadNet.getLaneId();
        String segmentId = roadNet.getSegmentId();

        if (!Objects.equals("null", crossId) && Objects.equals("null", rid) && Objects.equals("null", laneId)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 过滤拥堵场景下的事故上报
     * @param rid
     * @param trackIds
     * @return
     */
//    public static int ridIsCongested(String rid, String trackIds, ConcurrentHashMap<String, EventCross> cache){
//        int stopRidNum = 0;
//        if(StringUtils.isNotBlank(rid)){
//            for(EventCross e : cache.values()){
//                if(rid.equals(e.getRid())){
//                    if(!trackIds.contains(String.valueOf(e.getTrackId())) && e.getAreaDist() < 13.0){
//                        stopRidNum++;
//                    }
//                }
//            }
//        }
//        return stopRidNum;
//    }

    /**
     * 获取车道功能
     *
     * @param matchLaneName
     * @return
     */
    public static String getLaneFunc(String matchLaneName) {
        if (StringUtils.isBlank(matchLaneName)) {
            return Constant.STRAIGHT_LINE_CODE;
        }
        String[] s = matchLaneName.split(" ");
        String laneFuncChn = "";
        String laneFunc = "";
        if (s.length == 3) {
            laneFuncChn = s[1];
        }
        for (String k : Constant.mapTurn.keySet()) {
            String v = Constant.mapTurn.get(k);
            if (laneFuncChn.contains(v)) {
                laneFunc += k + Constant.COMMA;
            }
        }
        if (StringUtils.isNotBlank(laneFunc)) {
            laneFunc = laneFunc.substring(0, laneFunc.lastIndexOf(Constant.COMMA));
        } else {
            laneFunc = Constant.STRAIGHT_LINE_CODE;
            ;
        }
        return laneFunc;
    }

    /**
     * 排序 -- 时间戳 -- 升序
     * @param list
     * @return
     */
//    public static List<EventCross> getSortList(List<EventCross> list) {
//        try {
//            Collections.sort(list, new Comparator<EventCross>() {
//                @Override
//                public int compare(EventCross o1, EventCross o2) {
//                    long o1x = o1.getTime();
//                    long o2x = o2.getTime();
//                    if (o1x > o2x) {
//                        return 1;
//                    }
//                    if (o1x == o2x) {
//                        return 1;
//                    }
//                    return -1;
//                }
//            });
//            return list;
//        } catch (Exception e) {
//            return list;
//        }
//    }


    /**
     * 加载配置项
     */
    public String getWorkSpasePath() {
        String path = "";
        Properties props = System.getProperties();
        String osName = props.getProperty("os.name");
        String FLINK_WORKSPACE = System.getenv("FLINK_WORKSPACE_PATH");
        log.info("当前操作系统: " + osName + "  环境变量配置: " + FLINK_WORKSPACE);
        if (StringUtils.isNotBlank(osName) && osName.contains("Windows")) {
            path = Constant.PATH_WINDOWS;
            if (StringUtils.isNotBlank(FLINK_WORKSPACE)) {
                path = FLINK_WORKSPACE + "\\";
                log.info("读取 " + osName + " 外部环境变量配置: " + path);
            }
        } else {
            path = Constant.PATH_LINUX;
            if (StringUtils.isNotBlank(FLINK_WORKSPACE)) {
                path = FLINK_WORKSPACE;
                log.info("读取 " + osName + " 外部环境变量配置: " + path);
            }
        }
        return path;
    }

    /**
     * 根据方向角度计算RID 4方位 1：北 2：东 3：南 4：西
     */
    public static int getAngleDir(double angle) {
        int dir = 1;
        if (angle >= 0 && angle < 45) {
            dir = 1;
        } else if (angle >= 45 && angle < 135) {
            dir = 2;
        } else if (angle >= 135 && angle < 225) {
            dir = 3;
        } else if (angle >= 225 && angle < 315) {
            dir = 4;
        } else if (angle >= 315) {
            dir = 1;
        }
        return dir;
    }

    public static double getTravelDistance(List<CarTrackModel> carList) {
        String xys = carList.stream().map(o -> o.getLongitude() + "," + o.getLatitude()).collect(Collectors.joining(";"));
        double distance = 0;

        String tempx = "";
        String tempy = "";
        String[] sps = xys.split(";");

        int i = 0;
        for (String xy : sps) {
            String[] sps1 = xy.split(",");
            String x = sps1[0];
            String y = sps1[1];
            if (i > 1) {
                distance += GeomsConvertUtil.getDistance(Double.parseDouble(x), Double.parseDouble(y), Double.parseDouble(tempx), Double.parseDouble(tempy));
            }
            tempx = x;
            tempy = y;
            i++;
        }
        return distance;
    }
}
