package com.wanji.indicators.util;

import org.apache.commons.lang3.StringUtils;

import java.io.*;
import java.nio.charset.Charset;
import java.util.Properties;
import java.util.UUID;

public class PropertiesHelper implements Serializable {
  private Properties properties;
  
  private String bootstrapServers;
  
  private PropertiesHelper() {
    this.properties = getProperties();
  }
  
  private static class PropertiesHelperHolder {
    private static final PropertiesHelper propertiesHelper = new PropertiesHelper();
  }
  
  public static final PropertiesHelper getInstance() {
    return PropertiesHelperHolder.propertiesHelper;
  }
  
  public Properties getProperties() {
    if (this.properties == null) {
      this.properties = new Properties();
      try {
        this.bootstrapServers = System.getenv("KAFKA_BOOTSTRAP_SERVERS");
        //this.properties.load(PropertiesHelper.class.getResourceAsStream("/config.properties"));
        InputStream inputStream = PropertiesHelper.class.getResourceAsStream("/config.properties");
        BufferedReader bf = new BufferedReader(new InputStreamReader(inputStream, Charset.defaultCharset()));
        properties.load(bf);
      } catch (IOException e) {
        e.printStackTrace();
      } 
    } 
    return this.properties;
  }
  
  public synchronized Properties getConsumerProperties() {
    Properties consumerProps = new Properties();
    if (StringUtils.isNotBlank(this.bootstrapServers)) {
      consumerProps.setProperty("bootstrap.servers", this.bootstrapServers);
    } else {
      String kafka_bootstrap_servers = System.getenv("KAFKA_BOOTSTRAP_SERVERS");
      if (StringUtils.isNotBlank(kafka_bootstrap_servers)) {
        consumerProps.setProperty("bootstrap.servers", kafka_bootstrap_servers);
      } else {
        consumerProps.setProperty("bootstrap.servers", this.properties.getProperty("bootstrap.servers"));
      } 
    } 
    String s = UUID.randomUUID().toString().replace("-", "");
    consumerProps.setProperty("topic", this.properties.getProperty("consumer.topic"));
    consumerProps.setProperty("group.id", this.properties.getProperty("consumer.group.id"));
    consumerProps.setProperty("key-deserializer", this.properties.getProperty("consumer.key-deserializer"));
    consumerProps.setProperty("value-deserializer", this.properties.getProperty("consumer.value-deserializer"));
    return consumerProps;
  }

  public static Properties getConsumerProperties(Properties configProp) {
    Properties consumerProps = new Properties();
    consumerProps.setProperty("bootstrap.servers", configProp.getProperty("bootstrap.servers"));
    String s = UUID.randomUUID().toString().replace("-", "");
    consumerProps.setProperty("topic", configProp.getProperty("consumer.topic"));
    consumerProps.setProperty("group.id", configProp.getProperty("consumer.group.id"));
    consumerProps.setProperty("key-deserializer", configProp.getProperty("consumer.key-deserializer"));
    consumerProps.setProperty("value-deserializer", configProp.getProperty("consumer.value-deserializer"));
    return consumerProps;
  }
  
  public synchronized Properties getProducerProperties() {
    Properties producerProps = new Properties();
    if (StringUtils.isNotBlank(this.bootstrapServers)) {
      producerProps.setProperty("bootstrap.servers", this.bootstrapServers);
    } else {
      String kafka_bootstrap_servers = System.getenv("KAFKA_BOOTSTRAP_SERVERS");
      if (StringUtils.isNotBlank(kafka_bootstrap_servers)) {
        producerProps.setProperty("bootstrap.servers", kafka_bootstrap_servers);
      } else {
        producerProps.setProperty("bootstrap.servers", this.properties.getProperty("bootstrap.servers"));
      } 
    } 
    producerProps.setProperty("key-serializer", this.properties.getProperty("producer.key-serializer"));
    producerProps.setProperty("value-serializer", this.properties.getProperty("producer.value-serializer"));
    return producerProps;
  }

  public synchronized Properties getElasticProperties() {
    Properties esProps = new Properties();
    esProps.setProperty("elasticsearch.server", this.properties.getProperty("elasticsearch.server"));
    esProps.setProperty("elasticsearch.username", this.properties.getProperty("elasticsearch.username"));
    esProps.setProperty("elasticsearch.password", this.properties.getProperty("elasticsearch.password"));
    return esProps;
  }

}
